@@ -285,13 +285,18 @@ boot_script_parse_line (char *cmdline)
     /* Ignore comment line.  */
     return 0;
 
+#if 0
   if (*p && *p != ' ' && *p != '\t' && *p != '\n')
-    printf ("(bootstrap): %s\n", cmdline);
+    {
+      printf ("(bootstrap): %s\n", cmdline);
+    }
+#endif
 
   for (q = p; *q && *q != ' ' && *q != '\t' && *q != '\n'; q++)
     ;
   if (p == q)
-    return 0;
+      return 0;
+
   *q = '\0';
 
   /* Allocate a command structure.  */
@@ -490,6 +495,7 @@ boot_script_parse_line (char *cmdline)
 	    }
 	}
     }
+  
 
  bad:
   free_cmd (cmd, 1);
