@@ -1,6 +1,6 @@
 /* Inode management routines
 
-   Copyright (C) 1994, 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1994, 1995, 1996, 1997 Free Software Foundation, Inc.
 
    Converted for ext2fs by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -22,6 +22,19 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdio.h>
+#include <sys/stat.h>
+
+/* these flags aren't actually defined by a header file yet, so temporarily
+   disable them if necessary.  */
+#ifndef UF_APPEND
+#define UF_APPEND 0
+#endif
+#ifndef UF_NODUMP
+#define UF_NODUMP 0
+#endif
+#ifndef UF_IMMUTABLE
+#define UF_IMMUTABLE 0
+#endif
 
 #define	INOHSZ	512
 #if	((INOHSZ&(INOHSZ-1)) == 0)
@@ -32,7 +45,7 @@
 
 static struct node *nodehash[INOHSZ];
 
-static error_t read_disknode (struct node *np);
+static error_t read_node (struct node *np);
 
 spin_lock_t generation_lock = SPIN_LOCK_INITIALIZER;
 
@@ -76,7 +89,7 @@ diskfs_cached_lookup (int inum, struct node **npp)
   dn->dir_idx = 0;
   dn->pager = 0;
   rwlock_init (&dn->alloc_lock);
-  pokel_init (&dn->indir_pokel, disk_pager, disk_image);
+  pokel_init (&dn->indir_pokel, diskfs_disk_pager, disk_image);
 
   /* Create the new node.  */
   np = diskfs_make_node (dn);
@@ -94,7 +107,7 @@ diskfs_cached_lookup (int inum, struct node **npp)
   spin_unlock (&diskfs_node_refcnt_lock);
   
   /* Get the contents of NP off disk.  */
-  err = read_disknode (np);
+  err = read_node (np);
   
   if (!diskfs_check_readonly () && !np->dn_stat.st_gen)
     {
@@ -180,7 +193,7 @@ diskfs_new_hardrefs (struct node *np)
 
 /* Read stat information out of the ext2_inode. */
 static error_t
-read_disknode (struct node *np)
+read_node (struct node *np)
 {
   error_t err;
   static int fsid, fsidset;
@@ -193,8 +206,6 @@ read_disknode (struct node *np)
   if (err)
     return err;
 
-  np->istranslated = sblock->s_creator_os == EXT2_OS_HURD && di->i_translator;
-
   if (!fsidset)
     {
       fsid = getpid ();
@@ -221,23 +232,36 @@ read_disknode (struct node *np)
 #endif
 
   st->st_blocks = di->i_blocks;
-  st->st_flags = di->i_flags;
-  
+
+  st->st_flags = 0;
+  if (di->i_flags & EXT2_APPEND_FL)
+    st->st_flags |= UF_APPEND;
+  if (di->i_flags & EXT2_NODUMP_FL)
+    st->st_flags |= UF_NODUMP;
+  if (di->i_flags & EXT2_IMMUTABLE_FL)
+    st->st_flags |= UF_IMMUTABLE;
+
   if (sblock->s_creator_os == EXT2_OS_HURD)
     {
       st->st_mode = di->i_mode | (di->i_mode_high << 16);
+      st->st_mode &= ~S_ITRANS;
+      if (di->i_translator)
+	st->st_mode |= S_IPTRANS;
+
       st->st_uid = di->i_uid | (di->i_uid_high << 16);
       st->st_gid = di->i_gid | (di->i_gid_high << 16);
+
       st->st_author = di->i_author;
       if (st->st_author == -1)
 	st->st_author = st->st_uid;
     }
   else
     {
-      st->st_mode = di->i_mode;
+      st->st_mode = di->i_mode & ~S_ITRANS;
       st->st_uid = di->i_uid;
       st->st_gid = di->i_gid;
       st->st_author = st->st_uid;
+      np->author_tracks_uid = 1;
     }
 
   /* Setup the ext2fs auxiliary inode info.  */
@@ -288,10 +312,7 @@ read_disknode (struct node *np)
        allocated as well, although st_size may be zero for any type (cases
        where st_blocks=0 and st_size>0 include fast symlinks, and, under
        linux, some devices).  */
-    {
-      np->allocsize = 0;
-      assert (st->st_blocks == 0);
-    }
+    np->allocsize = 0;
 
   return 0;
 }
@@ -340,10 +361,22 @@ diskfs_validate_author_change (struct node *np, uid_t author)
   if (sblock->s_creator_os == EXT2_OS_HURD)
     return 0;
   else
-    /* For non-hurd filesystems, the auther & owner are the same.  */
+    /* For non-hurd filesystems, the author & owner are the same.  */
     return (author == np->dn_stat.st_uid) ? 0 : EINVAL;
 }
 
+/* The user may define this function.  Return 0 if NP's flags can be
+   changed to FLAGS; otherwise return an error code.  It must always
+   be possible to clear the flags.   */
+error_t
+diskfs_validate_flags_change (struct node *np, int flags)
+{
+  if (flags & (UF_NODUMP | UF_IMMUTABLE | UF_APPEND))
+    return 0;
+  else
+    return EINVAL;
+}
+
 /* Writes everything from NP's inode to the disk image, and returns a pointer
    to it, or NULL if nothing need be done.  */
 static struct ext2_inode *
@@ -375,14 +408,14 @@ write_node (struct node *np)
 
       /* Only the low 16 bits of these fields are standard across all ext2
 	 implementations.  */
-      di->i_mode = st->st_mode & 0xFFFF;
+      di->i_mode = st->st_mode & 0xFFFF & ~S_ITRANS;
       di->i_uid = st->st_uid & 0xFFFF;
       di->i_gid = st->st_gid & 0xFFFF;
 
       if (sblock->s_creator_os == EXT2_OS_HURD)
 	/* If this is a hurd-compatible filesystem, write the high bits too. */
 	{
-	  di->i_mode_high = (st->st_mode >> 16) & 0xffff;
+	  di->i_mode_high = (st->st_mode >> 16) & 0xffff & ~S_ITRANS;
 	  di->i_uid_high = st->st_uid >> 16;
 	  di->i_gid_high = st->st_gid >> 16;
 	  di->i_author = st->st_author;
@@ -393,7 +426,7 @@ write_node (struct node *np)
 	  assert ((st->st_uid & ~0xFFFF) == 0);
 	  assert ((st->st_gid & ~0xFFFF) == 0);
 	  assert ((st->st_mode & ~0xFFFF) == 0);
-	  assert (st->st_author == st->st_uid);
+	  assert (np->author_tracks_uid && st->st_author == st->st_uid);
 	}
 
       di->i_links_count = st->st_nlink;
@@ -409,9 +442,18 @@ write_node (struct node *np)
 #endif
 
       di->i_blocks = st->st_blocks;
-      di->i_flags = st->st_flags;
 
-      if (!np->istranslated && sblock->s_creator_os == EXT2_OS_HURD)
+      /* Convert generic flags in ST->st_flags to ext2-specific flags in DI
+         (but don't mess with ext2 flags we don't know about).  */
+      di->i_flags &= ~(EXT2_APPEND_FL | EXT2_NODUMP_FL | EXT2_IMMUTABLE_FL);
+      if (st->st_flags & UF_APPEND)
+	di->i_flags |= EXT2_APPEND_FL;
+      if (st->st_flags & UF_NODUMP)
+	di->i_flags |= EXT2_NODUMP_FL;
+      if (st->st_flags & UF_IMMUTABLE)
+	di->i_flags |= EXT2_IMMUTABLE_FL;
+
+      if (!(st->st_mode & S_IPTRANS) && sblock->s_creator_os == EXT2_OS_HURD)
 	di->i_translator = 0;
 
       /* Set dtime non-zero to indicate a deleted file.  */
@@ -449,7 +491,7 @@ diskfs_node_reload (struct node *node)
     }
   pokel_flush (&dn->indir_pokel);
   flush_node_pager (node);
-  read_disknode (node);
+  read_node (node);
 
   return 0;
 }
@@ -613,7 +655,7 @@ diskfs_set_translator (struct node *np, char *name, unsigned namelen,
       ext2_free_blocks (blkno, 1);
 
       np->dn_stat.st_blocks -= 1 << log2_stat_blocks_per_fs_block;
-      np->istranslated = 0;
+      np->dn_stat.st_mode &= ~S_IPTRANS;
       np->dn_set_ctime = 1;
     }
   
@@ -626,7 +668,7 @@ diskfs_set_translator (struct node *np, char *name, unsigned namelen,
       bcopy (buf, bptr (blkno), block_size);
       record_global_poke (bptr (blkno));
 
-      np->istranslated = 1;
+      np->dn_stat.st_mode |= S_IPTRANS;
       np->dn_set_ctime = 1;
     }
   
