@@ -1,5 +1,5 @@
 /* Definitions for fileserver helper functions
-   Copyright (C) 1994, 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1994, 1995, 1996, 1997 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -18,13 +18,17 @@
 #ifndef _HURD_DISKFS
 #define _HURD_DISKFS
 
-#include <argp.h>
 #include <assert.h>
 #include <unistd.h>
 #include <rwlock.h>
 #include <hurd/ports.h>
 #include <hurd/fshelp.h>
 #include <hurd/iohelp.h>
+#include <idvec.h>
+
+#ifndef DISKFS_EI
+#define DISKFS_EI extern inline
+#endif
 
 /* Each user port referring to a file points to one of these
    (with the aid of the ports library).  */
@@ -33,8 +37,7 @@ struct protid
   struct port_info pi;		/* libports info block */
 
   /* User identification */
-  uid_t *uids, *gids;
-  int nuids, ngids;
+  struct iouser *user;
 
   /* Object this refers to */
   struct peropen *po;
@@ -51,8 +54,16 @@ struct peropen
   int lock_status;
   int refcnt;
   int openstat;
-  mach_port_t dotdotport;	/* dotdot from ROOT through this peropen */
+
   struct node *np;
+
+  /* The parent of the translator's root node.  */
+  mach_port_t root_parent;
+
+  /* If this node is in a shadow tree, the parent of its root.  */
+  mach_port_t shadow_root_parent;
+  /* If in a shadow tree, its root node in this translator.  */
+  struct node *shadow_root;
 };
 
 /* A unique one of these exists for each node currently in use (and
@@ -65,7 +76,6 @@ struct node
   struct disknode *dn;
 
   struct stat dn_stat;
-  int istranslated;
 
   /* Stat has been modified if one of the following four fields
      is nonzero.  Also, if one of the dn_set_?time fields is nonzero,
@@ -95,6 +105,8 @@ struct node
   off_t allocsize;
 
   int cache_id;
+
+  int author_tracks_uid;
 };
 
 /* Possibly lookup types for diskfs_lookup call */
@@ -117,6 +129,7 @@ struct dirmod
 /* Special flag for diskfs_lookup. */
 #define SPEC_DOTDOT 0x10000000
 
+struct argp;			/* opaque in this file */
 
 /* Declarations of variables the library sets.  */
 
@@ -203,11 +216,13 @@ extern struct node *diskfs_root_node;
    filesystem server. */
 extern char *diskfs_server_name;
 
-/* The user must define these variables.  Set these to be the major, minor,
-   and edit version numbers.  */
-extern int diskfs_major_version;
-extern int diskfs_minor_version;
-extern int diskfs_edit_version;
+/* The user must define this variables.  Set this to be the server
+   version number.  */
+extern char *diskfs_server_version;
+
+/* The user may define this variable.  Set this to be any additional
+   version specification that should be printed for --version. */
+extern char *diskfs_extra_version;
 
 /* The user may define this variable.  This should be nonzero iff the
    filesystem format supports shortcutting symlink translation.
@@ -239,6 +254,13 @@ int diskfs_shortcut_ifsock;
    thread is started up (in diskfs_spawn_first_threa).   */
 extern int diskfs_default_sync_interval;
 
+/* The user must define this variable, which should be a string that somehow
+   identifies the particular disk this filesystem is interpreting.  It is
+   generally only used to print messages or to distinguish instances of the
+   same filesystem type from one another.  If this filesystem accesses no
+   external media, then define this to be 0.  */
+extern char *diskfs_disk_name;
+
 /* The user must define this function.  Set *STATFSBUF with
    appropriate values to reflect the current state of the filesystem.  */
 error_t diskfs_set_statfs (fsys_statfsbuf_t *statfsbuf);
@@ -364,7 +386,7 @@ error_t diskfs_grow (struct node *np, off_t size, struct protid *cred);
    metadata.  If CLEAN is nonzero, then after this is written the
    filesystem will be absolutely clean, and the non-paged metadata can
    so indicate.  */
-void diskfs_set_hypermetadata (int wait, int clean);
+error_t diskfs_set_hypermetadata (int wait, int clean);
 
 /* The user must define this function.  Allocate a new node to be of
    mode MODE in locked directory DP (don't actually set the mode or
@@ -512,33 +534,6 @@ error_t (*diskfs_read_symlink_hook)(struct node *np, char *target);
 
 /* The library exports the following functions for general use */
 
-/* Returns the name and a send right for the mach device on which the file
-   NAME is stored, and returns it in DEV_NAME (which is malloced) and PORT.
-   Other values returned are START, the first valid offset, SIZE, the the
-   number of blocks after START, and BLOCK_SIZE, the units in which the
-   device is addressed.
-
-   The device is opened for reading, and if the diskfs global variable
-   DISKFS_READONLY is false, writing.  */
-error_t diskfs_get_file_device (char *name,
-				char **dev_name, mach_port_t *port,
-				off_t *start, off_t *size, size_t *block_size);
-
-/* Returns a send right to for the mach device called NAME, and returns it in
-   PORT.  Other values returned are START, the first valid offset, SIZE, the
-   the number of blocks after START, and BLOCK_SIZE, the units in which the
-   device is addressed.
-
-   The device is opened for reading, and if the diskfs global variable
-   DISKFS_READ_ONLY is false, writing.
-
-   If NAME cannot be opened and this is a bootstrap filesystem, the user will
-   be prompted for new names until a valid one is found.  */
-error_t diskfs_get_mach_device (char *name,
-				mach_port_t *port,
-				off_t *start, off_t *size, size_t *block_size);
-
-
 /* Call this after arguments have been parsed to initialize the library.
    You must call this before calling any other diskfs functions, and after
    parsing diskfs options.  */
@@ -574,7 +569,7 @@ void diskfs_node_update (struct node *np, int wait);
 /* Add a hard reference to a node.  If there were no hard
    references previously, then the node cannot be locked
    (because you must hold a hard reference to hold the lock). */
-extern inline void
+DISKFS_EI void
 diskfs_nref (struct node *np)
 {
   int new_hardref;
@@ -593,7 +588,7 @@ diskfs_nref (struct node *np)
 /* Unlock node NP and release a hard reference; if this is the last
    hard reference and there are no links to the file then request
    soft references to be dropped.  */
-extern inline void
+DISKFS_EI void
 diskfs_nput (struct node *np)
 {
   int tried_drop_softrefs = 0;
@@ -643,7 +638,7 @@ diskfs_nput (struct node *np)
    hard reference in order to hold the lock).  If this is the last
    hard reference and there are no links, then request soft references
    to be dropped.  */
-extern inline void
+DISKFS_EI void
 diskfs_nrele (struct node *np)
 {
   int tried_drop_softrefs = 0;
@@ -683,7 +678,7 @@ diskfs_nrele (struct node *np)
 }
 
 /* Add a light reference to a node. */
-extern inline void
+DISKFS_EI void
 diskfs_nref_light (struct node *np)
 {
   spin_lock (&diskfs_node_refcnt_lock);
@@ -692,7 +687,7 @@ diskfs_nref_light (struct node *np)
 }
 
 /* Unlock node NP and release a light reference */
-extern inline void
+DISKFS_EI void
 diskfs_nput_light (struct node *np)
 {
   spin_lock (&diskfs_node_refcnt_lock);
@@ -710,7 +705,7 @@ diskfs_nput_light (struct node *np)
 /* Release a light reference on NP.  If NP is locked by anyone, then
    this cannot be the last reference (because you must hold a
    hard reference in order to hold the lock).  */
-extern inline void
+DISKFS_EI void
 diskfs_nrele_light (struct node *np)
 {
   spin_lock (&diskfs_node_refcnt_lock);
@@ -725,90 +720,6 @@ diskfs_nrele_light (struct node *np)
     spin_unlock (&diskfs_node_refcnt_lock);
 }
 
-/* Return nonzero iff the user identified by CRED has uid UID. */
-extern inline int
-diskfs_isuid (uid_t uid, struct protid *cred)
-{
-  int i;
-  for (i = 0; i < cred->nuids; i++)
-    if (cred->uids[i] == uid)
-      return 1;
-  return 0;
-}
-
-/* Return nonzero iff the user identified by CRED has group GRP. */
-extern inline int
-diskfs_groupmember (uid_t grp, struct protid *cred)
-{
-  int i;
-  for (i = 0; i < cred->ngids; i++)
-    if (cred->gids[i] == grp)
-      return 1;
-  return 0;
-}
-
-/* Check to see if the user identified by CRED is permitted to do
-   owner-only operations on node NP; if so, return 0; if not, return
-   EPERM. */
-extern inline error_t
-diskfs_isowner (struct node *np, struct protid *cred)
-{
-  /* Permitted if the user is the owner, superuser, or if the user
-     is in the group of the file and has the group ID as their user
-     ID.  (This last is colloquially known as `group leader'.) */
-  if (diskfs_isuid (np->dn_stat.st_uid, cred) || diskfs_isuid (0, cred)
-      || (diskfs_groupmember (np->dn_stat.st_gid, cred)
-	  && diskfs_isuid (np->dn_stat.st_gid, cred)))
-    return 0;
-  else
-    return EPERM;
-}
-
-/* Check to see is the user identified by CRED is permitted to do
-   operation OP on node NP.  Op is one of S_IREAD, S_IWRITE, or S_IEXEC.
-   Return 0 if the operation is permitted and EACCES if not. */
-extern inline error_t
-diskfs_access (struct node *np, int op, struct protid *cred)
-{
-  int gotit;
-  if (diskfs_isuid (0, cred))
-    gotit = 1;
-  else if (cred->nuids == 0 && (np->dn_stat.st_mode & S_IUSEUNK))
-    gotit = np->dn_stat.st_mode & (op << S_IUNKSHIFT);
-  else if (!diskfs_isowner (np, cred))
-    gotit = np->dn_stat.st_mode & op;
-  else if (diskfs_groupmember (np->dn_stat.st_gid, cred))
-    gotit = np->dn_stat.st_mode & (op >> 3);
-  else
-    gotit = np->dn_stat.st_mode & (op >> 6);
-  return gotit ? 0 : EACCES;
-}
-
-/* Check to see if the user identified by CRED is allowed to modify
-   directory DP with respect to existing file NP.  This is the same
-   as diskfs_access (dp, S_IWRITE, cred), except when the directory
-   has the sticky bit set.  (If there is no existing file NP, then
-   0 can be passed.)  */
-extern inline error_t
-diskfs_checkdirmod (struct node *dp, struct node *np,
-		    struct protid *cred)
-{
-  error_t err;
-
-  /* The user must be able to write the directory, but if the directory
-     is sticky, then the user must also be either the owner of the directory
-     or the file.  */
-  err = diskfs_access (dp, S_IWRITE, cred);
-  if (err)
-    return err;
-
-  if ((dp->dn_stat.st_mode & S_ISVTX) && np && !diskfs_isuid (0, cred)
-      && diskfs_isowner (dp, cred) && diskfs_isowner (np, cred))
-    return EACCES;
-
-  return 0;
-}
-
 /* Reading and writing of files. this is called by other filesystem
    routines and handles extension of files automatically.  NP is the
    node to be read or written, and must be locked.  DATA will be
@@ -937,25 +848,25 @@ diskfs_create_node (struct node *dir, char *name, mode_t mode,
 		    struct node **newnode, struct protid *cred,
 		    struct dirstat *ds);
 
-/* Create and return a protid for an existing peropen PO in CRED.  The uid
-   set is UID (length NUIDS); the gid set is GID (length NGIDS).  The node
-   PO->np must be locked. */
-error_t diskfs_create_protid (struct peropen *po, uid_t *uids, int nuids,
-			      uid_t *gids, int ngids, struct protid **cred);
+/* Create and return a protid for an existing peropen PO in CRED,
+   referring to user USER.  The node PO->np must be locked. */
+error_t diskfs_create_protid (struct peropen *po, struct iouser *user,
+			      struct protid **cred);
 
 /* Build and return in CRED a protid which has no user identification, for
    peropen PO.  The node PO->np must be locked.  */
 error_t diskfs_start_protid (struct peropen *po, struct protid **cred);
 
 /* Finish building protid CRED started with diskfs_start_protid;
-   the uid set is UID (length NUIDS); the gid set is GID (length NGIDS). */
-void diskfs_finish_protid (struct protid *cred, uid_t *uids, int nuids,
-			   gid_t *gids, int nguds);
+   the user to install is USER.  */
+void diskfs_finish_protid (struct protid *cred, struct iouser *user);
 
 /* Create and return a new peropen structure on node NP with open
-   flags FLAGS.  */
+   flags FLAGS.  The initial values for the root_parent, shadow_root, and
+   shadow_root_parent fields are copied from CONTEXT if it's non-zero,
+   otherwise zerod.  */
 struct peropen *diskfs_make_peropen (struct node *np, int flags,
-				     mach_port_t dotdotnode);
+				     struct peropen *context);
 
 /* Called when a protid CRED has no more references.  (Because references\
    to protids are maintained by the port management library, this is
@@ -1051,12 +962,10 @@ error_t diskfs_set_sync_interval (int interval);
    routine will parse them using DISKFS_RUNTIME_ARGP, which see.  */
 error_t diskfs_set_options (char *argz, size_t argz_len);
 
-/* Return an argz string describing the current options.  Fill *ARGZ
-   with a pointer to newly malloced storage holding the list and *LEN
-   to the length of that storage.  The default definition of this routine
-   simply initializes *ARGZ and *ARGZ_LEN to 0 and calls
-   diskfs_append_std_options.  */
-error_t diskfs_get_options (char **argz, unsigned *argz_len);
+/* Append to the malloced string *ARGZ of length *ARGZ_LEN a NUL-separated
+   list of the arguments to this translator.  The default definition of this
+   routine simply calls diskfs_append_std_options.  */
+error_t diskfs_append_args (char **argz, unsigned *argz_len);
 
 /* If this is defined or set to an argp structure, it will be used by the
    default diskfs_set_options to handle runtime option parsing.  The default
@@ -1071,7 +980,14 @@ extern const struct argp diskfs_std_runtime_argp;
 /* An argp structure for the standard diskfs command line arguments.  The
    user may call argp_parse on this to parse the command line, chain it onto
    the end of his own argp structure, or ignore it completely.  */
-extern const struct argp diskfs_std_startup_argp;
+extern const struct argp diskfs_startup_argp;
+
+/* An argp structure for the standard diskfs command line arguments plus a
+   store specification.  The address of a location in which to return the
+   resulting struct store_parsed structure should be passed as the input
+   argument to argp_parse; see the declaration for STORE_ARGP in
+   <hurd/store.h> for more information.  */
+extern const struct argp diskfs_store_startup_argp;
 
 /* *Appends* to ARGZ & ARGZ_LEN '\0'-separated options describing the standard
    diskfs option state (note that unlike diskfs_get_options, ARGZ & ARGZ_LEN
@@ -1094,62 +1010,7 @@ int diskfs_check_readonly (void);
    can be used by any user program that uses a mach device to hold the
    underlying filesystem.  */
 
-/* A pointer to an argp structure for the standard diskfs command line
-   arguments, that also parses a device argument.  The user may call
-   argp_parse on this to parse the command line, chain it onto the end of his
-   own argp structure, or ignore it completely.  */
-extern const struct argp diskfs_std_device_startup_argp;
-
-/* The following two are set by the preceding argument parser:  */
-/* The device specifier given on the command line.  */
-extern char *diskfs_device_arg;
-/* True if --machdev was specified, meaning that DISKFS_DEVICE_ARG names a
-   mach device and not a filesystem device node.  */
-extern int diskfs_use_mach_device;
-
-/* Uses the values of DISKFS_DEVICE_ARG and DISKFS_USE_MACH_DEVICE, and
-   attempts to open the device and set the values of DISKFS_DEVICE,
-   DISKFS_DEVICE_NAME, DISKFS_DEVICE_START, DISKFS_DEVICE_SIZE, and
-   DISKFS_DEVICE_BLOCK_SIZE.  */
-extern error_t diskfs_device_open ();
-
-/* A mach device port for the device we're using.  */
-extern mach_port_t diskfs_device;
-
-/* The mach device name of DISKFS_DEVICE.  May be 0 if unknown.  */
-extern char *diskfs_device_name;
-
-/* The first valid block of DISKFS_DEVICE, in units of
-   DISKFS_DEVICE_BLOCK_SIZE.  */
-extern off_t diskfs_device_start;
-
-/* The usable size of DISKFS_DEVICE, in units of DISKFS_DEVICE_BLOCK_SIZE.  */
-extern off_t diskfs_device_size;
-
-/* The unit of addressing for DISKFS_DEVICE.  */
-extern unsigned diskfs_device_block_size;
-
-/* Some handy calculations based on DISKFS_DEVICE_BLOCK_SIZE.  */
-/* Log base 2 of DEVICE_BLOCK_SIZE, or 0 if it's not a power of two.  */
-extern unsigned diskfs_log2_device_block_size;
-/* Log base 2 of the number of device blocks in a vm page.  This number is
-   only valid if DISKFS_LOG2_DEVICE_BLOCK_SIZE is not 0.  */
-extern unsigned diskfs_log2_device_blocks_per_page;
-
-/* Write disk block ADDR with DATA of LEN bytes to DISKFS_DEVICE, waiting for
-   completion.  ADDR is offset by DISKFS_DEVICE_START.  If an error occurs,
-   EIO is returned.  */
-error_t diskfs_device_write_sync (off_t addr, vm_address_t data, size_t len);
-
-/* Read disk block ADDR from DISKFS_DEVICE; put the address of the data in
-   DATA; read LEN bytes.  Always *DATA should be a full page no matter what.
-   ADDR is offset by DISKFS_DEVICE_START.  If an error occurs, EIO is
-   returned.  */
-error_t diskfs_device_read_sync (off_t addr, vm_address_t *data, size_t len);
-
 /* Make errors go somewhere reasonable.  */
 void diskfs_console_stdio ();
 
-
 #endif	/* hurd/diskfs.h */
-
