@@ -1,6 +1,6 @@
 /* Store I/O
 
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
 
    Written by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -24,10 +24,16 @@
 #define __STORE_H__
 
 #include <sys/types.h>
+#include <fcntl.h>
 
 #include <mach.h>
 #include <device/device.h>
 #include <hurd/hurd_types.h>
+
+#ifndef STORE_EI
+#define STORE_EI extern inline
+#endif
+
 
 /* A portion of a store.  If START == -1, it's a hole.  */
 struct store_run
@@ -78,7 +84,7 @@ struct store
   void *misc;			/* malloced */
   size_t misc_len;
 
-  struct store_class *class;
+  const struct store_class *class;
 
   /* A list of sub-stores.  The interpretation of this is type-specific.  */
   struct store **children;
@@ -101,20 +107,23 @@ struct store
 #define STORE_GENERIC_FLAGS	(STORE_READONLY | STORE_NO_FILEIO)
 
 /* Flags implemented by each backend.  */
-#define STORE_HARD_READONLY	0x0400	/* Can't be made writable.  */
-#define STORE_ENFORCED		0x0800	/* Range is enforced by device.  */
-#define STORE_BACKEND_SPEC_BASE	0x1000 /* Here up are backend-specific */
+#define STORE_HARD_READONLY	0x1000 /* Can't be made writable.  */
+#define STORE_ENFORCED		0x2000 /* Range is enforced by device.  */
+#define STORE_INACTIVE		0x4000 /* Not in a usable state.  */
+#define STORE_INNOCUOUS		0x8000 /* Cannot modify anything dangerous. */
+#define STORE_BACKEND_SPEC_BASE	0x10000 /* Here up are backend-specific */
 #define STORE_BACKEND_FLAGS	(STORE_HARD_READONLY | STORE_ENFORCED \
+				 | STORE_INACTIVE \
 				 | ~(STORE_BACKEND_SPEC_BASE - 1))
 
 typedef error_t (*store_write_meth_t)(struct store *store,
 				      off_t addr, size_t index,
-				      char *buf, mach_msg_type_number_t len,
+				      void *buf, mach_msg_type_number_t len,
 				      mach_msg_type_number_t *amount);
 typedef error_t (*store_read_meth_t)(struct store *store,
 				     off_t addr, size_t index,
 				     mach_msg_type_number_t amount,
-				     char **buf, mach_msg_type_number_t *len);
+				     void **buf, mach_msg_type_number_t *len);
 
 struct store_enc;		/* fwd decl */
 
@@ -142,7 +151,8 @@ struct store_class
 
   /* Decode from ENC a new store, which return in STORE.  CLASSES is used to
      lookup child classes.  */
-  error_t (*decode) (struct store_enc *enc, struct store_class *classes,
+  error_t (*decode) (struct store_enc *enc,
+		     const struct store_class *const *classes,
 		     struct store **store);
 
   /* Modify flags that reflect backend state, such as STORE_HARD_READONLY and
@@ -158,16 +168,42 @@ struct store_class
      made after all format-indendependent fields have been cloned.  */
   error_t (*clone) (const struct store *from, struct store *to);
 
-  /* For making a list of classes to pass to e.g. store_create.  */
-  struct store_class *next;
+  /* Return in STORE a store that only contains the parts of SOURCE as
+     enumerated in RUNS & RUNS_LEN, consuming SOURCE in the process.  The
+     default behavior, if REMAP is 0, is to replace SOURCE's run list with
+     the subset selected by RUNS, and return SOURCE.  */
+  error_t (*remap) (struct store *source,
+		    const struct store_run *runs, size_t num_runs,
+		    struct store **store);
+
+  /* Open a new store called NAME in this class.  CLASSES is supplied in case
+     it's desirable to open a sub-store in some manner.  */
+  error_t (*open) (const char *name, int flags,
+		   const struct store_class *const *classes,
+		   struct store **store);
+
+  /* Given a user argument ARG, this function should check it for syntactic
+     validaty, or print a syntax error, using ARGP_STATE in the normal
+     manner; if zero is returned, then this argument is assumed valid, and
+     can be passed to the open function.  If ARG is 0, then there were *no*
+     arguments specified; in this case, returning EINVAL means that this is
+     not kosher.  If PARSE is 0, then it is assumed that if this class has an
+     OPEN function, then validity can't be syntactically determined.  */
+  error_t (*validate_name) (const char *name,
+			    const struct store_class *const *classes);
 };
 
-/* Return a new store in STORE, which refers to the storage underlying
-   SOURCE.  CLASSES is used to select classes specified by the provider; if
-   it is 0, STORE_STD_CLASSES is used.  FLAGS is set with store_set_flags.  A
-   reference to SOURCE is created (but may be destroyed with
-   store_close_source).  */
-error_t store_create (file_t source, int flags, struct store_class *classes,
+
+/* Return a new store in STORE, which refers to the storage underlying SOURCE.
+   CLASSES is used to select classes specified by the provider; if it is 0,
+   STORE_STD_CLASSES is used.  FLAGS is set with store_set_flags, with the
+   exception of STORE_INACTIVE, which merely indicates that no attempt should
+   be made to activate an inactive store; if STORE_INACTIVE is not specified,
+   and the store returned for SOURCE is inactive, an attempt is made to
+   activate it (failure of which causes an error to be returned).  A reference
+   to SOURCE is created (but may be destroyed with store_close_source).  */
+error_t store_create (file_t source, int flags,
+		      const struct store_class *const *classes,
 		      struct store **store);
 
 void store_free (struct store *store);
@@ -177,13 +213,14 @@ void store_free (struct store *store);
    provider; if it is 0, STORE_STD_CLASSES is used.  FLAGS is set with
    store_set_flags.  A reference to the open file is created (but may be
    destroyed with store_close_source).  */
-error_t store_open (char *name, int flags, struct store_class *classes,
+error_t store_open (const char *name, int flags,
+		    const struct store_class *const *classes,
 		    struct store **store);
 
 /* Allocate a new store structure with class CLASS, and the various other
    fields initialized to the given parameters.  */
 struct store *
-_make_store (struct store_class *class, mach_port_t port, int flags,
+_make_store (const struct store_class *class, mach_port_t port, int flags,
 	     size_t block_size, const struct store_run *runs, size_t num_runs,
 	     off_t end);
 
@@ -205,23 +242,51 @@ error_t store_set_flags (struct store *store, int flags);
 /* Remove FLAGS from STORE's currently set flags.  */
 error_t store_clear_flags (struct store *store, int flags);
 
+/* Set FLAGS in all children of STORE, and if successfull, add FLAGS to
+   STORE's flags.  */
+error_t store_set_child_flags (struct store *store, int flags);
+
+/* Clear FLAGS in all children of STORE, and if successfull, remove FLAGS from
+   STORE's flags.  */
+error_t store_clear_child_flags (struct store *store, int flags);
+
+/* Returns true if STORE can safely be returned to a user who has accessed it
+   via a node using OPEN_FLAGS, without compromising security.  */
+STORE_EI int
+store_is_securely_returnable (struct store *store, int open_flags)
+{
+  int flags = store->flags;
+  return
+    (flags & (STORE_INNOCUOUS | STORE_INACTIVE))
+    || ((flags & STORE_ENFORCED)
+	&& (((open_flags & O_ACCMODE) == O_RDWR)
+	    || (flags & STORE_HARD_READONLY)));
+}
+
 /* Fills in the values of the various fields in STORE that are derivable from
    the set of runs & the block size.  */
 void _store_derive (struct store *store);
 
 /* Return in TO a copy of FROM.  */
 error_t store_clone (struct store *from, struct store **to);
+
+/* Return a store in STORE that reflects the blocks in RUNS & RUNS_LEN from
+   source; SOURCE is consumed, but not RUNS.  Unlike the store_remap_create
+   function, this may simply modify SOURCE and return it.  */
+error_t store_remap (struct store *source,
+		     const struct store_run *runs, size_t num_runs,
+		     struct store **store);
 
 /* Write LEN bytes from BUF to STORE at ADDR.  Returns the amount written in
    AMOUNT (in bytes).  ADDR is in BLOCKS (as defined by STORE->block_size).  */
 error_t store_write (struct store *store,
-		     off_t addr, char *buf, size_t len, size_t *amount);
+		     off_t addr, void *buf, size_t len, size_t *amount);
 
 /* Read AMOUNT bytes from STORE at ADDR into BUF & LEN (which following the
    usual mach buffer-return semantics) to STORE at ADDR.  ADDR is in BLOCKS
    (as defined by STORE->block_size).  Note that LEN is in bytes.  */
 error_t store_read (struct store *store,
-		    off_t addr, size_t amount, char **buf, size_t *len);
+		    off_t addr, size_t amount, void **buf, size_t *len);
 
 /* If STORE was created using store_create, remove the reference to the
    source from which it was created.  */
@@ -277,6 +342,18 @@ error_t _store_file_create (file_t file, int flags, size_t block_size,
 /* Open the file NAME, and return the corresponding store in STORE.  */
 error_t store_file_open (const char *name, int flags, struct store **store);
 
+/* Return a new store in STORE referring to the task TASK, consuming TASK.  */
+error_t store_task_create (task_t task, int flags, struct store **store);
+
+/* Like store_task_create, but doesn't query the task for information.  */
+error_t _store_task_create (task_t task, int flags, size_t block_size,
+			    const struct store_run *runs, size_t num_runs,
+			    struct store **store);
+
+/* Open the task NAME (NAME should be the task's pid), and return the
+   corresponding store in STORE.  */
+error_t store_task_open (const char *name, int flags, struct store **store);
+
 /* Return a new store in STORE that interleaves all the stores in STRIPES
    (NUM_STRIPES of them) every INTERLEAVE bytes; INTERLEAVE must be an
    integer multiple of each stripe's block size.  The stores in STRIPES are
@@ -292,24 +369,29 @@ error_t store_ileave_create (struct store * const *stripes, size_t num_stripes,
 error_t store_concat_create (struct store * const *stores, size_t num_stores,
 			     int flags, struct store **store);
 
-/* Return a new null store SIZE bytes long in STORE.  */
-error_t store_null_create (size_t size, int flags, struct store **store);
+/* Return a new store in STORE that reflects the blocks in RUNS & RUNS_LEN
+   from SOURCE; SOURCE is consumed, but RUNS is not.  Unlike the store_remap
+   function, this function always operates by creating a new store of type
+   `remap' which has SOURCE as a child, and so may be less efficient than
+   store_remap for some types of stores.  */
+error_t store_remap_create (struct store *source,
+			    const struct store_run *runs, size_t num_runs,
+			    int flags, struct store **store);
+
+/* Return a new zero store SIZE bytes long in STORE.  */
+error_t store_zero_create (size_t size, int flags, struct store **store);
 
 /* Standard store classes implemented by libstore.  */
-extern struct store_class *store_std_classes;
+extern const struct store_class *const store_std_classes[];
 
-/* Add CLASS to the list of standard classes.  It must not already be in the
-   list, or in any other, as its next field is simply written over.  */
-void _store_add_std_class (struct store_class *class);
-
-/* Use this macro to automagically add a class to STORE_STD_CLASSES at
-   startup.  */
-#define _STORE_STD_CLASS(class_struct)					    \
-static void _store_init_std_##class_struct () __attribute__ ((constructor));\
-static void _store_init_std_##class_struct ()				    \
-{									    \
-  _store_add_std_class (&class_struct);					    \
-}
+extern const struct store_class store_device_class;
+extern const struct store_class store_file_class;
+extern const struct store_class store_task_class;
+extern const struct store_class store_zero_class;
+extern const struct store_class store_ileave_class;
+extern const struct store_class store_concat_class;
+extern const struct store_class store_remap_class;
+extern const struct store_class store_query_class;
 
 /* Used to hold the various bits that make up the representation of a store
    for transmission via rpc.  See <hurd/hurd_types.h> for an explanation of
@@ -349,7 +431,23 @@ void store_enc_init (struct store_enc *enc,
 /* Deallocate storage used by the fields in ENC (but nothing is done with ENC
    itself).  */
 void store_enc_dealloc (struct store_enc *enc);
+
+/* Copy out the parameters from ENC into the given variables suitably for
+   returning from a file_get_storage_info rpc, and deallocate ENC.  */
+void store_enc_return (struct store_enc *enc,
+		       mach_port_t **ports, mach_msg_type_number_t *num_ports,
+		       int **ints, mach_msg_type_number_t *num_ints,
+		       off_t **offsets, mach_msg_type_number_t *num_offsets,
+		       char **data, mach_msg_type_number_t *data_len);
 
+/* Encode STORE into the given return variables, suitably for returning from a
+   file_get_storage_info rpc.  */
+error_t store_return (const struct store *store,
+		      mach_port_t **ports, mach_msg_type_number_t *num_ports,
+		      int **ints, mach_msg_type_number_t *num_ints,
+		      off_t **offsets, mach_msg_type_number_t *num_offsets,
+		      char **data, mach_msg_type_number_t *data_len);
+
 /* Encode STORE into ENC, which should have been prepared with
    store_enc_init, or return an error.  The contents of ENC may then be
    return as the value of file_get_storage_info; if for some reason this
@@ -361,7 +459,8 @@ error_t store_encode (const struct store *store, struct store_enc *enc);
    defines the mapping from hurd storage class ids to store classes; if it is
    0, STORE_STD_CLASSES is used.  If nothing else is to be done with ENC, its
    contents may then be freed using store_enc_dealloc.  */
-error_t store_decode (struct store_enc *enc, struct store_class *classes,
+error_t store_decode (struct store_enc *enc,
+		      const struct store_class *const *classes,
 		      struct store **store);
 
 /* Calls the allocate_encoding method in each child store of STORE,
@@ -378,9 +477,14 @@ error_t store_encode_children (const struct store *store,
 /* Decodes NUM_CHILDREN from ENC, storing the results into successive
    positions in CHILDREN.  */
 error_t store_decode_children (struct store_enc *enc, int num_children,
-			       struct store_class *classes,
+			       const struct store_class *const *classes,
 			       struct store **children);
 
+/* Call FUN with the vector RUNS of length NUM_RUNS extracted from ENC.  */
+error_t store_with_decoded_runs (struct store_enc *enc, size_t num_runs,
+				 error_t (*fun) (const struct store_run *runs,
+						 size_t num_runs));
+
 /* Standard encoding used for most leaf store types.  */
 error_t store_std_leaf_allocate_encoding (const struct store *store,
 					  struct store_enc *enc);
@@ -403,10 +507,22 @@ error_t store_std_leaf_decode (struct store_enc *enc,
 
 /* An argument parser that may be used for parsing a simple command line
    specification for stores.  The accompanying input parameter must be a
-   pointer to a location in which to store the resulting pointer to a struct
-   store_parsed.  */
+   pointer to a struct store_argp_params.  */
 extern struct argp store_argp;
 
+/* The structure used to pass args back and forth from STORE_ARGP.  */
+struct store_argp_params
+{
+  /* The resulting parsed result.  */
+  struct store_parsed *result;
+
+  /* If --store-type isn't specified use this; 0 is equivalent to "query".  */
+  const char *default_type;
+
+  /* The set of classes used to validate store-types and argument syntax. */
+  const struct store_class *const *classes;
+};
+
 /* The result of parsing a store, which should be enough information to open
    it, or return the arguments.  */
 struct store_parsed;
@@ -416,11 +532,26 @@ void store_parsed_free (struct store_parsed *parsed);
 
 /* Open PARSED, and return the corresponding store in STORE.  */
 error_t store_parsed_open (const struct store_parsed *parsed, int flags,
-			   struct store_class *classes,
 			   struct store **store);
 
 /* Add the arguments  PARSED, and return the corresponding store in STORE.  */
 error_t store_parsed_append_args (const struct store_parsed *parsed,
 				  char **args, size_t *args_len);
 
+/* Make a string describing PARSED, and return it in malloced storage in
+   NAME.  */
+error_t store_parsed_name (const struct store_parsed *parsed, char **name);
+
+/* XXX Hacks to avoid touching hurd/hurd_types.h; these should be merged into
+   that file the next time it is updated (in enum storage_types). XXX  */
+#define STORAGE_REMAP (STORAGE_LAYER + 1) /* new type */
+#define STORAGE_ZERO  STORAGE_NULL        /* renaming of STORAGE_NULL  */
+
+/* Doc for STORAGE_REMAP:
+   STORAGE_REMAP is a layer on top of another store that remaps its blocks
+   ...
+    remap  - 	     TY, FL, NR      	     NR * (OFFS, LEN)  -	 1
+      (BS and SIZE are that of the child)
+*/
+
 #endif /* __STORE_H__ */
