@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation
+   Copyright (C) 1993, 1994, 1995, 1996, 1997 Free Software Foundation
 
 This file is part of the GNU Hurd.
 
@@ -96,9 +96,8 @@ diskfs_start_bootstrap ()
 
   /* Create the port for current and root directory.  */
   err = diskfs_create_protid (diskfs_make_peropen (diskfs_root_node,
-						   O_READ | O_EXEC,
-						   MACH_PORT_NULL),
-			       0,0,0,0, &rootpi);
+						   O_READ | O_EXEC, 0),
+			      0, &rootpi);
   assert_perror (err);
   root_pt = ports_get_right (rootpi);
 
@@ -259,9 +258,8 @@ diskfs_S_exec_startup_get_info (mach_port_t port,
   *intarraylen = 0;
 
   err = diskfs_create_protid (diskfs_make_peropen (diskfs_root_node,
-						   O_READ | O_EXEC,
-						   MACH_PORT_NULL),
-			      0,0,0,0, &rootpi);
+						   O_READ | O_EXEC, 0),
+			      0, &rootpi);
   assert_perror (err);
   rootport = ports_get_right (rootpi);
   ports_port_deref (rootpi);
@@ -299,9 +297,8 @@ diskfs_execboot_fsys_startup (mach_port_t port, int flags,
 				diskfs_execboot_class)))
     return EOPNOTSUPP;
 
-  err = diskfs_create_protid (diskfs_make_peropen (diskfs_root_node, flags,
-						   MACH_PORT_NULL),
-			      0,0,0,0, &rootpi);
+  err = diskfs_create_protid (diskfs_make_peropen (diskfs_root_node, flags, 0),
+			      0, &rootpi);
   assert_perror (err);
   rootport = ports_get_right (rootpi);
   mach_port_insert_right (mach_task_self (), rootport, rootport,
@@ -413,9 +410,8 @@ diskfs_S_fsys_init (mach_port_t port,
   /* Get a port to the root directory to put in the library's
      data structures.  */
   err = diskfs_create_protid (diskfs_make_peropen (diskfs_root_node,
-						   O_READ|O_EXEC,
-						   MACH_PORT_NULL),
-			      0,0,0,0, &rootpi);
+						   O_READ|O_EXEC, 0),
+			      0, &rootpi);
   assert_perror (err);
   root_pt = ports_get_right (rootpi);
   ports_port_deref (rootpi);
