@@ -32,8 +32,7 @@ diskfs_S_io_duplicate (struct protid *cred,
   
   mutex_lock (&cred->po->np->lock);
 
-  err = diskfs_create_protid (cred->po, cred->uids, cred->nuids,
-			      cred->gids, cred->ngids, &newpi);
+  err = diskfs_create_protid (cred->po, cred->user, &newpi);
   if (! err)
     {
       *port = ports_get_right (newpi);
