@@ -52,8 +52,8 @@ diskfs_create_node (struct node *dir,
   np = *newnode;
 
   /* Initialize the on-disk fields. */
-  if (cred->nuids)
-    newuid = cred->uids[0];
+  if (cred->user->uids->num)
+    newuid = cred->user->uids->ids[0];
   else
     {
       newuid = dir->dn_stat.st_uid;
@@ -63,9 +63,11 @@ diskfs_create_node (struct node *dir,
   if (err)
     goto change_err;
   np->dn_stat.st_uid = newuid;
+  if (np->author_tracks_uid)
+    np->dn_stat.st_author = newuid;
 
   newgid = dir->dn_stat.st_gid;
-  if (!diskfs_groupmember (newgid, cred))
+  if (!idvec_contains (cred->user->gids, newgid))
     mode &= ~S_ISGID;
   err = diskfs_validate_group_change (np, newgid);
   if (err)
