@@ -1,5 +1,5 @@
 /* 
-   Copyright (C) 1995 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
    Written by Michael I. Bushnell, p/BSG.
 
    This file is part of the GNU Hurd.
@@ -27,7 +27,7 @@ netfs_S_dir_unlink (struct protid *user, char *name)
   error_t err;
   
   mutex_lock (&user->po->np->lock);
-  err = netfs_attempt_unlink (user->credential, user->po->np, name);
+  err = netfs_attempt_unlink (user->user, user->po->np, name);
   mutex_unlock (&user->po->np->lock);
   return err;
 }
