@@ -25,11 +25,13 @@ error_t
 netfs_S_file_check_access (struct protid *user,
 			   int *types)
 {
+  error_t err;
+  
   if (!user)
     return EOPNOTSUPP;
   
   mutex_lock (&user->po->np->lock);
-  netfs_report_access (user->credential, user->po->np, types);
+  err = netfs_report_access (user->user, user->po->np, types);
   mutex_unlock (&user->po->np->lock);
-  return 0;
+  return err;
 }
