@@ -1,5 +1,5 @@
 /* 
-   Copyright (C) 1995 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
    Written by Michael I. Bushnell, p/BSG.
 
    This file is part of the GNU Hurd.
@@ -34,7 +34,7 @@ netfs_S_file_syncfs (struct protid *user,
   /* Translators not yet supported by netfs. XXX */
 
   mutex_lock (&user->po->np->lock);
-  err = netfs_attempt_syncfs (user->credential, wait);
+  err = netfs_attempt_syncfs (user->user, wait);
   mutex_unlock (&user->po->np->lock);
   return err;
 }
