@@ -26,12 +26,14 @@ netfs_S_fsys_syncfs (mach_port_t cntl,
 		     int wait,
 		     int children)
 {
-  struct netcred *cred;
+  struct iouser *cred;
   uid_t root = 0;
   error_t err;
 
-  cred = netfs_make_credential (&root, 1, &root, 1);
+  cred = iohelp_create_iouser (make_idvec (), make_idvec ());
+  idvec_set_ids (cred->uids, &root, 1);
+  idvec_set_ids (cred->gids, &root, 1);
   err = netfs_attempt_syncfs (cred, wait);
-  netfs_drop_credential (cred);
+  iohelp_free_iouser (cred);
   return err;
 }
