@@ -1,6 +1,6 @@
 /* Access, formatting, & comparison routines for printing process info.
 
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
 
    Written by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -462,8 +462,11 @@ ps_emit_seconds (struct proc_stat *ps, struct ps_fmt_field *field,
 
   FG (field, void)(ps, &tv);
 
-  fmt_seconds (&tv, !(field->flags & PS_FMT_FIELD_AT_MOD), prec, ABS (width),
-	       buf, sizeof (buf));
+  if ((field->flags & PS_FMT_FIELD_COLON_MOD) && tv.tv_sec == 0)
+    strcpy (buf, "-");
+  else
+    fmt_seconds (&tv, !(field->flags & PS_FMT_FIELD_AT_MOD), prec, ABS (width),
+		 buf, sizeof (buf));
 
   return ps_stream_write_field (stream, buf, width);
 }
@@ -478,8 +481,11 @@ ps_emit_minutes (struct proc_stat *ps, struct ps_fmt_field *field,
 
   FG (field, int)(ps, &tv);
 
-  fmt_minutes (&tv, !(field->flags & PS_FMT_FIELD_AT_MOD), ABS (width),
-	       buf, sizeof (buf));
+  if ((field->flags & PS_FMT_FIELD_COLON_MOD) && tv.tv_sec < 60)
+    strcpy (buf, "-");
+  else
+    fmt_minutes (&tv, !(field->flags & PS_FMT_FIELD_AT_MOD), ABS (width),
+		 buf, sizeof (buf));
 
   return ps_stream_write_field (stream, buf, width);
 }
@@ -568,8 +574,8 @@ ps_emit_args (struct proc_stat *ps, struct ps_fmt_field *field,
 
   FG (field, void)(ps, &s0, &s0len);
 
-  if (s0 == NULL)
-    *buf = '\0';
+  if (!s0 || s0len == 0 )
+    strcpy (buf, "-");
   else
     {
       if (s0len > sizeof static_buf)
@@ -610,8 +616,8 @@ ps_emit_string (struct proc_stat *ps, struct ps_fmt_field *field,
 
   FG (field, void)(ps, &str, &len);
 
-  if (str == NULL)
-    str = "";
+  if (!str || len == 0)
+    str = "-";
 
   return ps_stream_write_trunc_field (stream, str, field->width);
 }
@@ -724,24 +730,31 @@ ps_emit_wait (struct proc_stat *ps, struct ps_fmt_field *field,
 
       /* See if we should give a more useful name for the port.  */
       if (strcmp (wait, "init#0") == 0)
-	wait = "cwdir";		/* Current directory */
+	wait = "cwd";		/* Current directory */
       else if (strcmp (wait, "init#1") == 0)
-	wait = "crdir";		/* Root directory */
+	wait = "root";		/* Root directory */
       else if (strcmp (wait, "init#2") == 0)
 	wait = "auth";		/* Auth port */
       else if (strcmp (wait, "init#3") == 0)
 	wait = "proc";		/* Proc port */
       else if (strcmp (wait, "init#4") == 0)
-	wait = "bootstrap";	/* Bootstrap port */
+	wait = "cttyid";	/* Ctty id port */
+      else if (strcmp (wait, "init#5") == 0)
+	wait = "boot";		/* Bootstrap port */
       else
-	/* See if we can shorten the name to fit better.  We happen know that
-	   all currently returned keys are unique in the first character. */
+	/* See if we can shorten the name to fit better.  */
 	{
-	  char *sep = index (wait, '#');
-	  if (sep && sep > wait)
+	  char *abbrev = 0, *num = 0;
+	  if (strncmp (wait, "fd#", 3) == 0)
+	    abbrev = "fd", num = wait + 3;
+	  else if (strncmp (wait, "bgfd#", 5) == 0)
+	    abbrev = "bg", num = wait + 5;
+	  else if (strncmp (wait, "port#", 5) == 0)
+	    abbrev = "", num = wait + 5;
+	  if (abbrev)
 	    {
 	      snprintf (port_name_buf, sizeof port_name_buf,
-			"%c%s", wait[0], sep);
+			"%s%s", abbrev, num);
 	      wait = port_name_buf;
 	    }
 	}
@@ -845,6 +858,16 @@ ps_nominal_zint (struct proc_stat *ps, const struct ps_getter *getter)
   return G (getter, int)(ps) == 0;
 }
 
+/* Neither is an empty string.  */
+int
+ps_nominal_string (struct proc_stat *ps, const struct ps_getter *getter)
+{
+  char *str;
+  size_t len;
+  G (getter, char *)(ps, &str, &len);
+  return !str || len == 0 || (len == 1 && *str == '-');
+}
+
 /* Priorities are similar, but have to be converted to the unix nice scale
    first.  */
 int
@@ -1051,9 +1074,9 @@ static const struct ps_fmt_spec specs[] =
   {"LColl",	0,	-5, -1, 0,
    &ps_login_col_getter,   ps_emit_int,     ps_cmp_ints,   0},
   {"Args",	0,	 0, -1, 0,
-   &ps_args_getter,	   ps_emit_args,    ps_cmp_strings,0},
+   &ps_args_getter,	   ps_emit_args,    ps_cmp_strings,ps_nominal_string},
   {"Arg0",	0,	0, -1, 0,
-   &ps_args_getter,	   ps_emit_string,  ps_cmp_strings,0},
+   &ps_args_getter,	   ps_emit_string,  ps_cmp_strings,ps_nominal_string},
   {"Time",	0,	-8, 2, 0,
    &ps_tot_time_getter,    ps_emit_seconds, ps_cmp_times,  0},
   {"UTime",	0,	-8, 2, 0,
