@@ -26,6 +26,11 @@
 /*
  * HISTORY
  * $Log: cprocs.c,v $
+ * Revision 1.9  1996/11/18 23:54:51  thomas
+ * Mon Nov 18 16:36:56 1996  Thomas Bushnell, n/BSG  <thomas@gnu.ai.mit.edu>
+ *
+ * 	* cprocs.c (cproc_create): Cast CHILD in assignment.
+ *
  * Revision 1.8  1995/12/06 19:48:34  mib
  * (condition_unimplies): Take address of (*impp)->next in assignment to
  * IMPP on loop step instruction.
@@ -752,7 +757,7 @@ cproc_create()
 					 sizeof (long int));
 		cproc_prepare(child, &child->context, stack);
 		/* Set up the cproc_self ptr at the base of CHILD's stack.  */
-		ur_cthread_ptr(stack) = child;
+		ur_cthread_ptr(stack) = (ur_cthread_t) child;
 		cproc_ready(child,0);
 	}
 	return child;
