@@ -1,5 +1,5 @@
 /* 
-   Copyright (C) 1994 Free Software Foundation
+   Copyright (C) 1994, 1997 Free Software Foundation
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -31,6 +31,19 @@ void _trivfs_end_using_protid (struct trivfs_protid *);
 struct trivfs_control *_trivfs_begin_using_control (mach_port_t);
 void _trivfs_end_using_control (struct trivfs_control *);
 
+/* Vectors of dynamically allocated port classes/buckets.  */
+
+/* Protid port classes.  */
+extern struct port_class **trivfs_dynamic_protid_port_classes;
+extern size_t trivfs_num_dynamic_protid_port_classes;
+
+/* Control port classes.  */
+extern struct port_class **trivfs_dynamic_control_port_classes;
+extern size_t trivfs_num_dynamic_control_port_classes;
+
+/* Port buckets.  */
+extern struct port_bucket **trivfs_dynamic_port_buckets;
+extern size_t trivfs_num_dynamic_port_buckets;
+
 #define TRIVFS_PRIV_H_INCLUDED
 #endif
-
