@@ -18,8 +18,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA. */
 
-#include "nfs.h"
-
+#define malloc a_byte_for_every_bozotic_sun_lossage_and_youd_need_a_lotta_ram
+#include <rpc/types.h>
+#undef TRUE			/* Get rid of sun defs.  */
+#undef FALSE
+#undef malloc
 #include <rpc/pmap_prot.h>
 #include <errno.h>
 #include <sys/socket.h>
@@ -28,6 +31,7 @@
 #include <netinet/in.h>
 #include <stdio.h>
 
+#include "nfs.h"
 #include "mount.h"
 
 /* Service name for portmapper */
