@@ -1,5 +1,5 @@
 /* Data structures and global variables for NFS client
-   Copyright (C) 1994, 1995, 1996 Free Software Foundation
+   Copyright (C) 1994, 1995, 1996, 1997 Free Software Foundation
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -15,10 +15,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
-
-/* Needed for the rpcsvc include files to work. */
-typedef int bool_t;		/* Ick. */
-
 #include <sys/stat.h>
 #include <sys/types.h>
 #include "nfs-spec.h"
@@ -75,14 +71,6 @@ struct netnode
   char *dead_name;
 };
 
-/* Credential structure to identify a particular user. */
-struct netcred
-{
-  uid_t *uids, *gids;
-  int nuids, ngids;
-  int refcnt;
-};
-
 /* Socket file descriptor for talking to RPC servers. */
 int main_udp_socket;
 
@@ -159,10 +147,6 @@ extern int protocol_version;
 #define INTSIZE(len) (((len)+3)>>2)
 
 
-/* cred.c */
-int cred_has_uid (struct netcred *, uid_t);
-int cred_has_gid (struct netcred *, gid_t);
-
 /* nfs.c */
 int hurd_mode_to_nfs_type (mode_t);
 int *xdr_encode_fhandle (int *, struct fhandle *);
@@ -173,10 +157,10 @@ int *xdr_encode_sattr_ids (int *, u_int, u_int);
 int *xdr_encode_sattr_size (int *, off_t);
 int *xdr_encode_sattr_times (int *, struct timespec *, struct timespec *);
 int *xdr_encode_sattr_stat (int *, struct stat *);
-int *xdr_encode_create_state (int *, mode_t);
+int *xdr_encode_create_state (int *, mode_t, uid_t);
 int *xdr_decode_fattr (int *, struct stat *);
 int *xdr_decode_string (int *, char *);
-int *nfs_initialize_rpc (int, struct netcred *, size_t, void **, 
+int *nfs_initialize_rpc (int, struct iouser *, size_t, void **, 
 			 struct node *, uid_t);
 error_t nfs_error_trans (int);
 
