@@ -1,5 +1,5 @@
 /* Libnetfs callbacks for node operations in NFS client
-   Copyright (C) 1994, 1995, 1996 Free Software Foundation
+   Copyright (C) 1994, 1995, 1996, 1997 Free Software Foundation
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -132,7 +132,7 @@ process_wcc_stat (struct node *np, int *p, int mod)
 /* Implement the netfs_validate_stat callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_validate_stat (struct node *np, struct netcred *cred)
+netfs_validate_stat (struct node *np, struct iouser *cred)
 {
   int *p;
   void *rpcbuf;
@@ -142,7 +142,7 @@ netfs_validate_stat (struct node *np, struct netcred *cred)
     return 0;
 
   p = nfs_initialize_rpc (NFSPROC_GETATTR (protocol_version),
-			  (struct netcred *) -1, 0, &rpcbuf, np, -1);
+			  (struct iouser *) -1, 0, &rpcbuf, np, -1);
   p = xdr_encode_fhandle (p, &np->nn->handle);
   
   err = conduct_rpc (&rpcbuf, &p);
@@ -160,7 +160,7 @@ netfs_validate_stat (struct node *np, struct netcred *cred)
 /* Implement the netfs_attempt_chown callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_chown (struct netcred *cred, struct node *np,
+netfs_attempt_chown (struct iouser *cred, struct node *np,
 		     uid_t uid, gid_t gid)
 {
   int *p;
@@ -190,7 +190,7 @@ netfs_attempt_chown (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_chauthor callback as described in
    <hurd/netfs.h>.  */
 error_t
-netfs_attempt_chauthor (struct netcred *cred, struct node *rp,
+netfs_attempt_chauthor (struct iouser *cred, struct node *rp,
 			uid_t author)
 {
   return EOPNOTSUPP;
@@ -199,7 +199,7 @@ netfs_attempt_chauthor (struct netcred *cred, struct node *rp,
 /* Implement the netfs_attempt_chmod callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_chmod (struct netcred *cred, struct node *np,
+netfs_attempt_chmod (struct iouser *cred, struct node *np,
 		     mode_t mode)
 {
   int *p;
@@ -263,7 +263,7 @@ netfs_attempt_chmod (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_chflags callback as described in
    <hurd/netfs.h>. */
 error_t 
-netfs_attempt_chflags (struct netcred *cred, struct node *np,
+netfs_attempt_chflags (struct iouser *cred, struct node *np,
 		       int flags)
 {
   return EOPNOTSUPP;
@@ -272,7 +272,7 @@ netfs_attempt_chflags (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_utimes callback as described in
    <hurd/netfs.h>.  */
 error_t
-netfs_attempt_utimes (struct netcred *cred, struct node *np,
+netfs_attempt_utimes (struct iouser *cred, struct node *np,
 		      struct timespec *atime, struct timespec *mtime)
 {
   int *p;
@@ -301,7 +301,7 @@ netfs_attempt_utimes (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_set_size callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_set_size (struct netcred *cred, struct node *np,
+netfs_attempt_set_size (struct iouser *cred, struct node *np,
 			off_t size)
 {
   int *p;
@@ -323,6 +323,23 @@ netfs_attempt_set_size (struct netcred *cred, struct node *np,
 	p = process_wcc_stat (np, p, !err);
     }
   
+  /* If we got EACCES, but the user has the file open for writing,
+     then the NFS protocol has screwed us.  There's nothing we can do,
+     except in the important case of opens with
+     O_TRUNC|O_CREAT|O_WRONLY|O_EXCL where the new mode does not allow
+     writing.  RCS, for example, uses this to create lock files.  So permit
+     cases where the O_TRUNC isn't doing anything to succeed if the user
+     does have the file open for writing.  */
+  if (err == EACCES)
+    {
+      err = netfs_validate_stat (np, cred);
+      if (!err && np->nn_stat.st_size == size)
+	err = 0;
+      else
+	/* Never mind, put the old error back */
+	err = EACCES;
+    }
+
   free (rpcbuf);
   return err;
 }
@@ -330,7 +347,7 @@ netfs_attempt_set_size (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_statfs callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_statfs (struct netcred *cred, struct node *np,
+netfs_attempt_statfs (struct iouser *cred, struct node *np,
 		      struct statfs *st)
 {
   int *p;
@@ -346,18 +363,15 @@ netfs_attempt_statfs (struct netcred *cred, struct node *np,
   
   if (!err)
     {
-      long iosize, bsize;
-      iosize = *p++;
-      bsize = *p++;
-      st->f_bsize = iosize;
-      st->f_blocks = ((*p++) * bsize) / iosize;
-      st->f_bfree = ((*p++) * bsize) / iosize;
-      st->f_bavail = ((*p++) * bsize) / iosize;
-      
+      p++;			/* skip IOSIZE field */
+      st->f_bsize = ntohl (*p++);
+      st->f_blocks = ntohl (*p++);
+      st->f_bfree = ntohl (*p++);
+      st->f_bavail = ntohl (*p++);
       st->f_type = FSTYPE_NFS;
       st->f_files = 0;
       st->f_ffree = 0;
-      st->f_fsid = 0;	/* XXX wrong  */
+      st->f_fsid = getpid ();
       st->f_namelen = 0;
     }
   
@@ -368,7 +382,7 @@ netfs_attempt_statfs (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_sync callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_sync (struct netcred *cred, struct node *np, int wait)
+netfs_attempt_sync (struct iouser *cred, struct node *np, int wait)
 {
   /* We are already completely synchronous. */
   return 0;
@@ -377,7 +391,7 @@ netfs_attempt_sync (struct netcred *cred, struct node *np, int wait)
 /* Implement the netfs_attempt_syncfs callback as described in
    <hurd/netfs.h>.  */
 error_t
-netfs_attempt_syncfs (struct netcred *cred, int wait)
+netfs_attempt_syncfs (struct iouser *cred, int wait)
 {
   return 0;
 }
@@ -385,7 +399,7 @@ netfs_attempt_syncfs (struct netcred *cred, int wait)
 /* Implement the netfs_attempt_read callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_read (struct netcred *cred, struct node *np,
+netfs_attempt_read (struct iouser *cred, struct node *np,
 		    off_t offset, size_t *len, void *data)
 {
   int *p;
@@ -452,7 +466,7 @@ netfs_attempt_read (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_write callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_write (struct netcred *cred, struct node *np,
+netfs_attempt_write (struct iouser *cred, struct node *np,
 		     off_t offset, size_t *len, void *data)
 {
   int *p;
@@ -505,6 +519,13 @@ netfs_attempt_write (struct netcred *cred, struct node *np,
 	    }
 	}
       
+      if (err == EINTR && amt != *len)
+	{
+	  *len -= amt;
+	  free (rpcbuf);
+	  return 0;
+	}
+      
       if (err)
 	{
 	  *len = 0;
@@ -517,7 +538,7 @@ netfs_attempt_write (struct netcred *cred, struct node *np,
 
 /* See if NAME exists in DIR for CRED.  If so, return EEXIST.  */
 error_t
-verify_nonexistent (struct netcred *cred, struct node *dir,
+verify_nonexistent (struct iouser *cred, struct node *dir,
 		    char *name)
 {
   int *p;
@@ -544,7 +565,7 @@ verify_nonexistent (struct netcred *cred, struct node *dir,
 /* Implement the netfs_attempt_lookup callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_lookup (struct netcred *cred, struct node *np,
+netfs_attempt_lookup (struct iouser *cred, struct node *np,
 		      char *name, struct node **newnp)
 {
   int *p;
@@ -591,26 +612,44 @@ netfs_attempt_lookup (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_mkdir callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_mkdir (struct netcred *cred, struct node *np,
+netfs_attempt_mkdir (struct iouser *cred, struct node *np,
 		     char *name, mode_t mode)
 {
   int *p;
   void *rpcbuf;
   error_t err;
-  
+  uid_t owner;
+  struct node *newnp;
+
+  if (cred->uids->num)
+    owner = cred->uids->ids[0];
+  else
+    {
+      err = netfs_validate_stat (np, cred);
+      owner = err ? 0 : np->nn_stat.st_uid;
+      mode &= ~S_ISUID;
+    }
+
   p = nfs_initialize_rpc (NFSPROC_MKDIR (protocol_version),
 			  cred, 0, &rpcbuf, np, -1);
   p = xdr_encode_fhandle (p, &np->nn->handle);
   p = xdr_encode_string (p, name);
-  p = xdr_encode_create_state (p, mode);
+  p = xdr_encode_create_state (p, mode, owner);
   
   err = conduct_rpc (&rpcbuf, &p);
   if (!err)
     err = nfs_error_trans (ntohl (*p++));
   
-  /* Ignore returned information */
-  /* XXX should probably cache it */
+  p = lookup_fhandle (p, &newnp);
+  p = process_returned_stat (newnp, p, 1);
 
+  /* Did we set the owner correctly?  If not, try, but ignore failures. */
+  if (!netfs_validate_stat (newnp, -1) && newnp->nn_stat.st_uid != owner)
+    netfs_attempt_chown (-1, newnp, owner, newnp->nn_stat.st_gid);
+
+  /* We don't actually return this. */
+  netfs_nput (newnp);
+  
   free (rpcbuf);
   return err;
 }
@@ -618,7 +657,7 @@ netfs_attempt_mkdir (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_rmdir callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_rmdir (struct netcred *cred, struct node *np,
+netfs_attempt_rmdir (struct iouser *cred, struct node *np,
 		     char *name)
 {
   int *p;
@@ -647,7 +686,7 @@ netfs_attempt_rmdir (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_link callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_link (struct netcred *cred, struct node *dir,
+netfs_attempt_link (struct iouser *cred, struct node *dir,
 		    struct node *np, char *name, int excl)
 {
   int *p;
@@ -880,7 +919,7 @@ netfs_attempt_link (struct netcred *cred, struct node *dir,
       mutex_unlock (&np->lock);
       
       mutex_lock (&dir->lock);
-      netfs_attempt_unlink ((struct netcred *)-1, dir, name);
+      netfs_attempt_unlink ((struct iouser *)-1, dir, name);
       mutex_unlock (&dir->lock);
     }
   else
@@ -892,7 +931,7 @@ netfs_attempt_link (struct netcred *cred, struct node *dir,
 /* Implement the netfs_attempt_mkfile callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_mkfile (struct netcred *cred, struct node *dir,
+netfs_attempt_mkfile (struct iouser *cred, struct node *dir,
 		      mode_t mode, struct node **newnp)
 {
   error_t err;
@@ -931,12 +970,22 @@ netfs_attempt_mkfile (struct netcred *cred, struct node *dir,
 /* Implement the netfs_attempt_create_file callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_create_file (struct netcred *cred, struct node *np,
+netfs_attempt_create_file (struct iouser *cred, struct node *np,
 			   char *name, mode_t mode, struct node **newnp)
 {
   int *p;
   void *rpcbuf;
   error_t err;
+  uid_t owner;
+
+  if (cred->uids->num)
+    owner = cred->uids->ids[0];
+  else
+    {
+      err = netfs_validate_stat (np, cred);
+      owner = err ? 0 : np->nn_stat.st_uid;
+      mode &= ~S_ISUID;
+    }
 
   /* RFC 1094 says that create is always exclusive.  But Sun doesn't
      actually *implement* the spec.  No, of course not.  So we have to do
@@ -945,7 +994,10 @@ netfs_attempt_create_file (struct netcred *cred, struct node *np,
     {
       err = verify_nonexistent (cred, np, name);
       if (err)
-	return err;
+	{
+	  mutex_unlock (&np->lock);
+	  return err;
+	}
     }
 
   p = nfs_initialize_rpc (NFSPROC_CREATE (protocol_version),
@@ -963,7 +1015,7 @@ netfs_attempt_create_file (struct netcred *cred, struct node *np,
       p++;
     }
   else
-    p = xdr_encode_create_state (p, mode);
+    p = xdr_encode_create_state (p, mode, owner);
   
   err = conduct_rpc (&rpcbuf, &p);
 
@@ -989,6 +1041,10 @@ netfs_attempt_create_file (struct netcred *cred, struct node *np,
 	  if (*newnp)
 	    mutex_lock (&(*newnp)->lock);
 	}
+
+      if (*newnp && !netfs_validate_stat (*newnp, -1)
+	  && (*newnp)->nn_stat.st_uid != owner)
+	netfs_attempt_chown (-1, *newnp, owner, (*newnp)->nn_stat.st_gid);
     }
   else
     *newnp = 0;
@@ -1000,7 +1056,7 @@ netfs_attempt_create_file (struct netcred *cred, struct node *np,
 /* Implement the netfs_attempt_unlink callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_unlink (struct netcred *cred, struct node *dir,
+netfs_attempt_unlink (struct iouser *cred, struct node *dir,
 		      char *name)
 {
   int *p;
@@ -1079,7 +1135,7 @@ netfs_attempt_unlink (struct netcred *cred, struct node *dir,
 /* Implement the netfs_attempt_rename callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_rename (struct netcred *cred, struct node *fromdir,
+netfs_attempt_rename (struct iouser *cred, struct node *fromdir,
 		      char *fromname, struct node *todir, char *toname, 
 		      int excl)
 {
@@ -1121,7 +1177,7 @@ netfs_attempt_rename (struct netcred *cred, struct node *fromdir,
 /* Implement the netfs_attempt_readlink callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_readlink (struct netcred *cred, struct node *np,
+netfs_attempt_readlink (struct iouser *cred, struct node *np,
 			char *buf)
 {
   int *p;
@@ -1152,59 +1208,15 @@ netfs_attempt_readlink (struct netcred *cred, struct node *np,
   return err;
 }
 
-/* For an NFS node NODE, guess whether CRED is able to read or write
-   it by hoping the server uses permissions bits in the "expected
-   way".  Return the or of O_READ, O_WRITE, and O_EXEC accordingly as
-   each is possible. */
-static int
-guess_mode_use (struct node *np, 
-		struct netcred *cred)
-{
-  error_t err;
-
-  err = netfs_validate_stat (np, cred);
-  if (err)
-    return err;
-  
-  if (cred_has_uid (cred, 0))
-    return O_READ|O_WRITE|O_EXEC;
-  else if (cred->nuids == 0
-	   && (np->nn_stat.st_mode & S_IUSEUNK))
-    return 
-      (((np->nn_stat.st_mode & 04000000) ? O_READ : 0)
-       | ((np->nn_stat.st_mode & 02000000) ? O_WRITE : 0)
-       | ((np->nn_stat.st_mode & 01000000) ? O_EXEC : 0));
-  else if (cred_has_uid (cred, np->nn_stat.st_uid)
-	   || (cred_has_gid (cred, np->nn_stat.st_gid)
-	       && cred_has_uid (cred, np->nn_stat.st_gid)))
-    /* Owner */
-    return 
-      (((np->nn_stat.st_mode & 0400) ? O_READ : 0)
-       | ((np->nn_stat.st_mode & 0200) ? O_WRITE : 0)
-       | ((np->nn_stat.st_mode & 0100) ? O_EXEC : 0));
-  else if (cred_has_gid (cred, np->nn_stat.st_gid))
-    /* Group */
-    return 
-      (((np->nn_stat.st_mode & 040) ? O_READ : 0)
-       | ((np->nn_stat.st_mode & 020) ? O_WRITE : 0)
-       | ((np->nn_stat.st_mode & 010) ? O_EXEC : 0));
-  else
-    /* Other */
-    return 
-      (((np->nn_stat.st_mode & 4) ? O_READ : 0)
-       | ((np->nn_stat.st_mode & 2) ? O_WRITE : 0)
-       | ((np->nn_stat.st_mode & 1) ? O_EXEC : 0));
-}
-
 /* Implement the netfs_check_open_permissions callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_check_open_permissions (struct netcred *cred, struct node *np,
+netfs_check_open_permissions (struct iouser *cred, struct node *np,
 			      int flags, int newnode)
 {
   int modes;
   
-  if ((flags & (O_READ|O_WRITE|O_EXEC)) == 0)
+  if (newnode || (flags & (O_READ|O_WRITE|O_EXEC)) == 0)
     return 0;
   
   netfs_report_access (cred, np, &modes);
@@ -1216,13 +1228,29 @@ netfs_check_open_permissions (struct netcred *cred, struct node *np,
 
 /* Implement the netfs_report_access callback as described in
    <hurd/netfs.h>. */
-void
-netfs_report_access (struct netcred *cred,
+error_t
+netfs_report_access (struct iouser *cred,
 		     struct node *np,
 		     int *types)
 {
+  error_t err;
+  
+  err = netfs_validate_stat (np, cred);
+  if (err)
+    return err;
+  
   if (protocol_version == 2)
-    *types = guess_mode_use (np, cred);
+    {
+      /* Hope the server means the same thing be the bits as we do. */
+      *types = 0;
+      if (fshelp_access (&np->nn_stat, S_IREAD, cred) == 0)
+	*types |= O_READ;
+      if (fshelp_access (&np->nn_stat, S_IWRITE, cred) == 0)
+	*types |= O_WRITE;
+      if (fshelp_access (&np->nn_stat, S_IEXEC, cred) == 0)
+	*types |= O_EXEC;
+      return 0;
+    }
   else
     {
       int *p;
@@ -1231,9 +1259,6 @@ netfs_report_access (struct netcred *cred,
       int ret;
       int write_check, execute_check;
 
-      err = netfs_validate_stat (np, cred);
-      if (err)
-	goto fallback;
       if (S_ISDIR (np->nn_stat.st_mode))
 	{
 	  write_check = ACCESS3_MODIFY | ACCESS3_DELETE | ACCESS3_EXTEND;
@@ -1261,11 +1286,8 @@ netfs_report_access (struct netcred *cred,
 			| (ret & write_check ? O_WRITE : 0)
 			| (ret & execute_check ? O_EXEC : 0));
 	    }
-	  else
-	    /* fall back, sigh. */
-          fallback:
-	    *types = guess_mode_use (np, cred);
 	}
+      return err;
     }
 }
 
@@ -1275,7 +1297,7 @@ netfs_report_access (struct netcred *cred,
 /* Implement the netfs_check_open_permissions callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_check_open_permissions (struct netcred *cred, struct node *np,
+netfs_check_open_permissions (struct iouser *cred, struct node *np,
 			      int flags, int newnode)
 {
   char byte;
@@ -1365,7 +1387,7 @@ netfs_check_open_permissions (struct netcred *cred, struct node *np,
 /* Implement the netfs_report_access callback as described in
    <hurd/netfs.h>. */
 void
-netfs_report_access (struct netcred *cred,
+netfs_report_access (struct iouser *cred,
 		     struct node *np,
 		     int *types)
 {
@@ -1415,7 +1437,7 @@ netfs_report_access (struct netcred *cred,
    the error code.  Set BUFSIZEP to the amount of data used inside
    *BUFP and TOTALENTRIES to the total number of entries copied. */
 static error_t
-fetch_directory (struct netcred *cred, struct node *dir,
+fetch_directory (struct iouser *cred, struct node *dir,
 		 void **bufp, size_t *bufsizep, int *totalentries)
 {
   void *buf;
@@ -1513,7 +1535,7 @@ fetch_directory (struct netcred *cred, struct node *dir,
 /* Implement the netfs_get_directs callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_get_dirents (struct netcred *cred, struct node *np,
+netfs_get_dirents (struct iouser *cred, struct node *np,
 		   int entry, int nentries, char **data,
 		   mach_msg_type_number_t *datacnt,
 		   vm_size_t bufsiz, int *amt)
@@ -1583,7 +1605,7 @@ netfs_get_dirents (struct netcred *cred, struct node *np,
 /* Implement the netfs_set_translator callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_set_translator (struct netcred *cred,
+netfs_set_translator (struct iouser *cred,
 		      struct node *np,
 		      char *argz,
 		      size_t argzlen)
@@ -1594,7 +1616,7 @@ netfs_set_translator (struct netcred *cred,
 /* Implement the netfs_attempt_mksymlink callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_mksymlink (struct netcred *cred,
+netfs_attempt_mksymlink (struct iouser *cred,
 			 struct node *np,
 			 char *arg)
 {
@@ -1614,7 +1636,7 @@ netfs_attempt_mksymlink (struct netcred *cred,
 /* Implement the netfs_attempt_mkdev callback as described in
    <hurd/netfs.h>. */
 error_t
-netfs_attempt_mkdev (struct netcred *cred,
+netfs_attempt_mkdev (struct iouser *cred,
 		     struct node *np,
 		     mode_t type,
 		     dev_t indexes)
