@@ -1,6 +1,6 @@
 /* Hurd-aware fsck wrapper
 
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
 
    Written by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -55,7 +55,7 @@
 
 #include "fstab.h"
 
-char *argp_program_version = STANDARD_HURD_VERSION (fsck);
+const char *argp_program_version = STANDARD_HURD_VERSION (fsck);
 
 
 /* for debugging  */
@@ -274,7 +274,10 @@ fsck_cleanup (struct fsck *fsck, int remount, int make_writable)
   error_t err = 0;
   struct fs *fs = fsck->fs;
 
-  *fsck->self = fsck->next;	/* Remove from chain.  */
+  /* Remove from chain.  */
+  *fsck->self = fsck->next;
+  if (fsck->next)
+    fsck->next->self = fsck->self;
 
   fs_debug (fs, "Cleaning up after fsck (remount = %d, make_writable = %d)",
 	    remount, make_writable);
@@ -423,8 +426,11 @@ fsck (struct fstab *fstab, int flags, int max_parallel)
 		  merge_status (fscks_wait (&fscks));
 		merge_status (fscks_start_fsck (&fscks, fs));
 	      }
-	    else
+	    else if (autom)
 	      fs_debug (fs, "Not fsckable");
+	    else
+	      error (0, 0, "%s: %s: Not a fsckable filesystem type",
+		     fs->mntent.mnt_dir, fs->mntent.mnt_type);
 	  }
 
       /* Now wait for them all to finish.  */
