@@ -1,5 +1,5 @@
 /* 
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
    Written by Michael I. Bushnell, p/BSG.
 
    This file is part of the GNU Hurd.
@@ -28,6 +28,7 @@
 #include <cthreads.h>
 #include <hurd.h>
 #include <stdio.h>
+#include <hurd/iohelp.h>
 #include "ourmsg_U.h"
 
 
@@ -104,15 +105,15 @@ init_users ()
 
 static error_t
 open_hook (struct trivfs_control *cntl,
-	   uid_t *uids, u_int nuids,
-	   uid_t *gids, u_int ngids,
+	   struct iouser *user,
 	   int flags)
 {
+  static int open_count = 0;	/* XXX debugging */
   int cancel = 0;
   error_t err;
   
   if (cntl == ptyctl)
-    return pty_open_hook (cntl, uids, nuids, gids, ngids, flags);
+    return pty_open_hook (cntl, user, flags);
 
   if ((flags & (O_READ|O_WRITE)) == 0)
     return 0;
@@ -133,18 +134,28 @@ open_hook (struct trivfs_control *cntl,
   
       bcopy (ttydefchars, termstate.c_cc, NCCS);
 
+      bzero (&window_size, sizeof window_size);
+
       termflags |= NO_OWNER;
     }
-  else if (termflags & EXCL_USE)
+  else
     {
-      mutex_unlock (&global_lock);
-      return EBUSY;
+      assert (open_count > 0);	/* XXX debugging */
+
+      if (termflags & EXCL_USE)
+	{
+	  mutex_unlock (&global_lock);
+	  return EBUSY;
+	}
     }
 
-  /* Wait for carrier to turn on. */
-  while (((termflags & NO_CARRIER) && !(termstate.c_cflag & CLOCAL))
-	 && !(flags & O_NONBLOCK)
-	 && !cancel)
+  open_count++;			/* XXX debugging */
+
+  /* XXX debugging */
+  assert (! (termstate.c_oflag & OTILDE));
+
+  /* Assert DTR if necessary. */
+  if ((termflags & NO_CARRIER) && !(termstate.c_cflag & CLOCAL))
     {
       err = (*bottom->assert_dtr) ();
       if (err)
@@ -152,14 +163,14 @@ open_hook (struct trivfs_control *cntl,
 	  mutex_unlock (&global_lock);
 	  return err;
 	}
-      cancel = hurd_condition_wait (&carrier_alert, &global_lock);
     }
+  
+  /* Wait for carrier to turn on. */
+  while (((termflags & NO_CARRIER) && !(termstate.c_cflag & CLOCAL))
+	 && !(flags & O_NONBLOCK)
+	 && !cancel)
+    cancel = hurd_condition_wait (&carrier_alert, &global_lock);
       
-  if ((termflags & NO_CARRIER) && !(termstate.c_cflag & CLOCAL))
-    {
-      mutex_unlock (&global_lock);
-      return EWOULDBLOCK;
-    }
   if (cancel)
     {
       mutex_unlock (&global_lock);
@@ -167,15 +178,13 @@ open_hook (struct trivfs_control *cntl,
     }
   
   termflags |= TTY_OPEN;
-  if (!(termstate.c_cflag & CIGNORE))
-    (*bottom->set_bits) ();
-
+  (*bottom->set_bits) ();
 
   mutex_unlock (&global_lock);
   return 0;
 }
-error_t (*trivfs_check_open_hook) (struct trivfs_control *, uid_t *,
-				   u_int, uid_t *, u_int, int)
+error_t (*trivfs_check_open_hook) (struct trivfs_control *, 
+				   struct iouser *, int)
      = open_hook;
 
 static error_t
@@ -291,7 +300,6 @@ trivfs_modify_stat (struct trivfs_protid *cred, struct stat *st)
   st->st_fstype = FSTYPE_TERM;
   st->st_fsid = getpid ();
   st->st_ino = 0;
-  st->st_mode &= ~S_IFMT;
   st->st_mode = term_mode;
   st->st_uid = term_owner;
   st->st_gid = term_group;
@@ -346,7 +354,9 @@ S_termctty_open_terminal (mach_port_t arg,
 
   if (!err)
     {
-      err = trivfs_open (termctl, 0, 0, 0, 0, flags, new_realnode, &newcred);
+      err = trivfs_open (termctl, 
+			 iohelp_create_iouser (make_idvec (), make_idvec ()),
+			 flags, new_realnode, &newcred);
       if (!err)
 	{
 	  *result = ports_get_right (newcred);
@@ -420,56 +430,40 @@ trivfs_S_file_chown (struct trivfs_protid *cred,
 		     uid_t uid,
 		     gid_t gid)
 {
-  int i;
-  int noticed_uid;
-
-  /* This routine is flawed in several ways; it needs to
-     be rewritted once the idvec handling stuff can do
-     permission checks. */
-
+  struct stat st;
+  error_t err;
+  
   if (!cred)
     return EOPNOTSUPP;
 
-  noticed_uid = 0;
   mutex_lock (&global_lock);
-  for (i = 0; i < cred->nuids; i++)
-    {
-      if (cred->uids[i] == uid)
-	noticed_uid = 1;
-      if (cred->uids[i] == 0 || cred->uids[i] == term_owner)
-	{
-	  /* Make sure UID is legitimate */
-	  if (!cred->isroot && !noticed_uid && term_owner != uid)
-	    {
-	      /* Continue scanning UIDS */
-	      for (i++; i < cred->nuids; i++)
-		if (cred->uids[i] == uid)
-		  noticed_uid = 1;
-	      if (!noticed_uid)
-		{
-		  mutex_unlock (&global_lock);
-		  return EPERM;
-		}
-	    }
-	  
-	  /* Make sure GID is legitimate */
-	  for (i = 0; i < cred->ngids || cred->isroot; i++)
-	    if (cred->isroot || cred->gids[i] == gid)
-	      {
-		/* Make the change */
-		term_owner = uid;
-		term_group = gid;
-		mutex_unlock (&global_lock);
-		return 0;
-	      }
 
-	  /* Not legitimate */
-	  break;
+  /* XXX */
+  st.st_uid = term_owner;
+  st.st_gid = term_group;
+  
+  if (!cred->isroot)
+    {
+      err = fshelp_isowner (&st, cred->user);
+      if (err)
+	goto out;
+
+      if (!idvec_contains (cred->user->uids, uid)
+	  || !idvec_contains (cred->user->gids, gid))
+	{
+	  err = EPERM;
+	  goto out;
 	}
     }
 
+  /* Make the change */
+  term_owner = uid;
+  term_group = gid;
+  err = 0;
+  
+out:
   mutex_unlock (&global_lock);
-  return EPERM;
+  return err;
 }
 
 /* Implement chmod locally */
@@ -479,41 +473,40 @@ trivfs_S_file_chmod (struct trivfs_protid *cred,
 		     mach_msg_type_name_t reply_type,
 		     mode_t mode)
 {
-  int i;
+  error_t err;
+  struct stat st;
   
   if (!cred)
     return EOPNOTSUPP;
   
   mutex_lock (&global_lock);
-  for (i = 0; i < cred->nuids; i++)
-    if (cred->isroot || cred->uids[i] == term_owner)
-      {
-	if (!cred->isroot)
-	  {
-	    mode &= S_ISVTX;
+  if (!cred->isroot)
+    {
+      /* XXX */
+      st.st_uid = term_owner;
+      st.st_gid = term_group;
+      
+      err = fshelp_isowner (&st, cred->user);
+      if (err)
+	goto out;
 
-	    for (i = 0; i < cred->nuids; i++)
-	      if (cred->uids[i] == term_owner)
-		break;
-	    if (i == cred->nuids)
-	      mode &= ~S_ISUID;
-	    
-	    for (i = 0; i < cred->ngids; i++)
-	      if (cred->gids[i] == term_group)
-		break;
-	    if (i == cred->nuids)
-	      mode &= ~S_ISGID;
-	  }
-	
-	term_mode = (mode | S_IFCHR);
-	mutex_unlock (&global_lock);
-	return 0;
-      }
+      mode &= S_ISVTX;
+      
+      if (!idvec_contains (cred->user->uids, term_owner))
+	mode &= ~S_ISUID;
+      
+      if (!idvec_contains (cred->user->gids, term_group))
+	mode &= ~S_ISUID;
+    }
+
+  term_mode = ((mode & ~S_IFMT & ~S_ITRANS & ~S_ISPARE) | S_IFCHR | S_IROOT);
+  err = 0;
+  
+out:
   mutex_unlock (&global_lock);
-  return EPERM;
+  return err;
 }
 
-
 error_t
 trivfs_S_file_check_access (struct trivfs_protid *cred,
 			    mach_port_t reply, mach_msg_type_name_t reply_type,
@@ -1071,8 +1064,8 @@ set_state (io_t port,
       else
 	termstate.c_lflag &= ~EXTPROC;
 
-      if (external_processing || !(termstate.c_cflag & CIGNORE))
-	(*bottom->set_bits) ();
+      (*bottom->set_bits) ();
+
       if (oldlflag & ICANON)
 	{
 	  if (!(termstate.c_lflag & ICANON))
@@ -1229,7 +1222,6 @@ S_tioctl_tiocswinsz (io_t port,
       return EOPNOTSUPP;
     }
 
-
   mutex_lock (&global_lock);
   
   if (!(cred->po->openmodes & (O_READ|O_WRITE)))
@@ -1239,10 +1231,11 @@ S_tioctl_tiocswinsz (io_t port,
 
   ports_port_deref (cred);
 
-  if (!err && (size.ws_row - window_size.ws_row +
-	       size.ws_col - window_size.ws_col +
-	       size.ws_xpixel - window_size.ws_xpixel +
-	       size.ws_ypixel - window_size.ws_ypixel) != 0)
+  if (! err
+      && (size.ws_row != window_size.ws_row
+	  || size.ws_col != window_size.ws_col
+	  || size.ws_xpixel != window_size.ws_xpixel
+	  || size.ws_ypixel != window_size.ws_ypixel))
     {
       /* The size is actually changing.  Record the new size and notify the
 	 process group.  */
