@@ -1,5 +1,5 @@
 /* Inode management routines
-   Copyright (C) 1994, 1995, 1996 Free Software Foundation
+   Copyright (C) 1994, 1995, 1996, 1997 Free Software Foundation
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -21,6 +21,7 @@
 #include <stdio.h>
 #include <netinet/in.h>
 #include <fcntl.h>
+#include <hurd/store.h>
 
 #define	INOHSZ	512
 #if	((INOHSZ&(INOHSZ-1)) == 0)
@@ -220,9 +221,7 @@ read_disknode (struct node *np)
   if (err)
     return err;
 
-  np->istranslated = !! di->di_trans;
-
-  if (!fsidset)
+  if (! fsidset)
     {
       fsid = getpid ();
       fsidset = 1;
@@ -233,8 +232,10 @@ read_disknode (struct node *np)
   st->st_ino = np->dn->number;
   st->st_gen = read_disk_entry (di->di_gen);
   st->st_rdev = read_disk_entry(di->di_rdev);
-  st->st_mode = (read_disk_entry (di->di_model) 
-		 | (read_disk_entry (di->di_modeh) << 16));
+  st->st_mode = (((read_disk_entry (di->di_model) 
+		   | (read_disk_entry (di->di_modeh) << 16))
+		  & ~S_ITRANS)
+		 | (di->di_trans ? S_IPTRANS : 0));
   st->st_nlink = read_disk_entry (di->di_nlink);
   st->st_size = read_disk_entry (di->di_size);
 #ifdef notyet
@@ -257,7 +258,8 @@ read_disknode (struct node *np)
     {
       st->st_uid = read_disk_entry (di->di_ouid);
       st->st_gid = read_disk_entry (di->di_ogid);
-      st->st_author = 0;
+      st->st_author = st->st_uid;
+      np->author_tracks_uid = 1;
     }
   else
     {
@@ -299,6 +301,18 @@ error_t diskfs_node_reload (struct node *node)
   return 0;
 }
 
+/* Return 0 if NP's author can be changed to AUTHOR; otherwise return an
+   error code. */
+error_t
+diskfs_validate_author_change (struct node *np, uid_t author)
+{
+  if (compat_mode == COMPAT_GNU)
+    return 0;
+  else
+    /* For non-hurd filesystems, the author & owner are the same.  */
+    return (author == np->dn_stat.st_uid) ? 0 : EINVAL;
+}
+
 static void
 write_node (struct node *np)
 {
@@ -325,12 +339,13 @@ write_node (struct node *np)
 
       if (compat_mode == COMPAT_GNU)
 	{
-	  write_disk_entry (di->di_model, st->st_mode & 0xffff);
-	  write_disk_entry (di->di_modeh, (st->st_mode >> 16) & 0xffff);
+	  mode_t mode = st->st_mode & ~S_ITRANS;
+	  write_disk_entry (di->di_model, mode & 0xffff);
+	  write_disk_entry (di->di_modeh, (mode >> 16) & 0xffff);
 	}
       else 
 	{
-	  write_disk_entry (di->di_model, st->st_mode & 0xffff);
+	  write_disk_entry (di->di_model, st->st_mode & 0xffff & ~S_ITRANS);
 	  di->di_modeh = 0;
 	}
 
@@ -556,7 +571,7 @@ diskfs_set_translator (struct node *np, char *name, u_int namelen,
       di->di_trans = 0;
       record_poke (di, sizeof (struct dinode));
       np->dn_stat.st_blocks -= btodb (sblock->fs_bsize);
-      np->istranslated = 0;
+      np->dn_stat.st_mode &= ~S_IPTRANS;
       np->dn_set_ctime = 1;
     }
   
@@ -568,7 +583,7 @@ diskfs_set_translator (struct node *np, char *name, u_int namelen,
       bcopy (buf, disk_image + fsaddr (sblock, blkno), sblock->fs_bsize);
       sync_disk_blocks (blkno, sblock->fs_bsize, 1);
 
-      np->istranslated = 1;
+      np->dn_stat.st_mode |= S_IPTRANS;
       np->dn_set_ctime = 1;
     }
   
@@ -621,229 +636,74 @@ diskfs_shutdown_soft_ports ()
    offset into inode block holding inode (4 bytes) */
 error_t
 diskfs_S_file_get_storage_info (struct protid *cred,
-				int *class,
-				off_t **addresses,
-				u_int *naddresses,
-				size_t *block_size,
-				char *storage_name,
-				mach_port_t *storage_port,
-				mach_msg_type_name_t *storage_port_type,
-				char **storage_data,
-				u_int *storage_data_len,
-				int *flags)
+				mach_port_t **ports,
+				mach_msg_type_name_t *ports_type,
+				mach_msg_type_number_t *num_ports,
+				int **ints, mach_msg_type_number_t *num_ints,
+				off_t **offsets,
+				mach_msg_type_number_t *num_offsets,
+				char **data, mach_msg_type_number_t *data_len)
 {
   error_t err;
   struct node *np;
-  int i;
-  struct dinode *di;
-  void *cp;
+  struct store *file_store;
+  struct store_run runs[NDADDR];
+  size_t num_runs = 0;
   
+  if (! cred)
+    return EOPNOTSUPP;
+
   np = cred->po->np;
   mutex_lock (&np->lock);
   
   /* See if this file fits in the direct block pointers.  If not, punt
      for now.  (Reading indir blocks is a pain, and I'm postponing
      pain.)  XXX */
-
   if (np->allocsize > NDADDR * sblock->fs_bsize)
     {
       mutex_unlock (&np->lock);
       return EINVAL;
     }
-  
-  if (*naddresses < NDADDR * 2)
-    vm_allocate (mach_task_self (), (vm_address_t *) addresses, 
-		 sizeof (int) * NDADDR * 2, 1);
-  else
-    bzero (addresses, *naddresses * 2 * sizeof (int));
-  *naddresses = NDADDR * 2;
 
-  if (*storage_data_len < 4 * sizeof (int))
-    vm_allocate (mach_task_self (), (vm_address_t *) storage_data, 
-		 sizeof (int) * 4, 1);
-  *storage_data_len = 4 * sizeof (int);
-
-  di = dino (np->dn->number);
-  
   err = diskfs_catch_exception ();
-  if (err)
-    {
-      mutex_unlock (&np->lock);
-      return err;
-    }
-  
-  /* Copy the block pointers */
+  if (! err)
+    if (!direct_symlink_extension
+	|| np->dn_stat.st_size >= sblock->fs_maxsymlinklen
+	|| !S_ISLNK (np->dn_stat.st_mode))
+      /* Copy the block pointers */
+      {
+	int i;
+	struct store_run *run = runs;
+	struct dinode *di = dino (np->dn->number);
 
-  if (!direct_symlink_extension
-      || np->dn_stat.st_size >= sblock->fs_maxsymlinklen
-      || !S_ISLNK (np->dn_stat.st_mode))
-    {
-      for (i = 0; i < NDADDR; i++)
-	{
-	  (*addresses)[2 * i] = fsbtodb (sblock, 
-					 read_disk_entry (di->di_db[i]));
-	  if ((i + 1) * sblock->fs_bsize > np->allocsize)
-	    (*addresses)[2 * i + 1] = np->allocsize - i * sblock->fs_bsize;
-	  else
-	    (*addresses)[2 * i + 1] = sblock->fs_bsize;
-	}
-    }
-
-  /* Fill in the aux data */
-  cp = *storage_data;
-
-  *(int *)cp = htonl (np->dn->number);
-  cp += sizeof (int);
-  
-  *(int *)cp = htonl (di->di_trans);
-  cp += sizeof (int);
-  
-  *(int *)cp = htonl (fsbtodb (sblock, ino_to_fsba (sblock, np->dn->number)));
-  cp += sizeof (int);
-  
-  *(int *)cp = htonl (ino_to_fsbo (sblock, np->dn->number)
-		      * sizeof (struct dinode));
-  
-      
+	for (i = 0; i < NDADDR; i++)
+	  {
+	    off_t start = fsbtodb (sblock, read_disk_entry (di->di_db[i]));
+	    off_t length = 
+	      (((i + 1) * sblock->fs_bsize > np->allocsize)
+	       ? np->allocsize - i * sblock->fs_bsize
+	       : sblock->fs_bsize);
+	    if (num_runs == 0 || run->start + run->length != start)
+	      *run++ = (struct store_run){ start, length };
+	    else
+	      run->length += length;
+	  }
+      }
   diskfs_end_catch_exception ();
-  
-  *class = STORAGE_DEVICE;
-  *flags = 0;
-  *block_size = DEV_BSIZE;
 
-  if (diskfs_device_name)
-    strcpy (storage_name, diskfs_device_name);
-  
-  if (diskfs_isuid (0, cred))
-    *storage_port = diskfs_device;
-  else
-    *storage_port = MACH_PORT_NULL;
-  *storage_port_type = MACH_MSG_TYPE_COPY_SEND;
-  
   mutex_unlock (&np->lock);
   
-  return 0;
-}
-
-/* Must be exactly 28 bytes long */
-struct ufs_fhandle
-{
-  int filler1;
-  ino_t inum;
-  long gen;
-  int filler2[4];
-};
-
-/* Return an NFS file handle */
-
-error_t
-diskfs_S_file_getfh (struct protid *cred,
-		     char **fh,
-		     u_int *fh_len)
-{
-  struct node *np;
-  error_t err;
-  struct ufs_fhandle *f;
-
-  if (!cred)
-    return EOPNOTSUPP;
-
-  if (!diskfs_isuid (0, cred))
-    return EPERM;
-  
-  np = cred->po->np;
-
-  mutex_lock (&np->lock);
-
-  if (*fh_len < sizeof (struct ufs_fhandle))
-    vm_allocate (mach_task_self (), (vm_address_t *) fh, 
-		 sizeof (struct ufs_fhandle), 1);
-  *fh_len = sizeof (struct ufs_fhandle);
-  
-  f = (struct ufs_fhandle *) *fh;
-  f->inum = np->dn->number;
-  f->gen = np->dn_stat.st_gen;
-  f->filler1 = 0;
-  f->filler2[0] = f->filler2[1] = f->filler2[2] = f->filler2[3] = 0;
-  mutex_unlock (&np->lock);
-  return 0;
-}
-
-/* Lookup an NFS file handle */
-error_t
-diskfs_S_fsys_getfile (mach_port_t fsys,
-		       mach_port_t reply,
-		       mach_msg_type_name_t replytype,
-		       uid_t *uids,
-		       u_int nuids,
-		       uid_t *gids,
-		       u_int ngids,
-		       char *handle,
-		       u_int handlelen,
-		       mach_port_t *file,
-		       mach_msg_type_name_t *filetype)
-{
-  struct port_info *pt = ports_lookup_port (diskfs_port_bucket, fsys,
-					    diskfs_control_class);
-  struct node *np;
-  struct ufs_fhandle *f;
-  error_t err;
-  int flags;
-  struct protid fakecred, *newpi;
-  
-  if (!pt)
-    return EOPNOTSUPP;
-
-  if (handlelen != sizeof (struct ufs_fhandle))
+  if (! err)
+    err = store_clone (store, &file_store);
+  if (! err)
     {
-      ports_port_deref (pt);
-      return EINVAL;
+      err = store_remap (file_store, runs, num_runs, &file_store);
+      if (! err)
+	err = store_return (file_store, ports, num_ports, ints, num_ints,
+			    offsets, num_offsets, data, data_len);
+      store_free (file_store);
     }
+  *ports_type = MACH_MSG_TYPE_COPY_SEND;
 
-  f = (struct ufs_fhandle *) handle;
-
-  err = diskfs_cached_lookup (f->inum, &np);
-  if (err)
-    {
-      ports_port_deref (pt);
-      return err;
-    }
-
-  if (np->dn_stat.st_gen != f->gen)
-    {
-      diskfs_nput (np);
-      ports_port_deref (pt);
-      return ESTALE;
-    }
-  
-  /* This call should have a flags arg, but until then... */
-  fakecred.uids = uids;
-  fakecred.gids = gids;
-  fakecred.nuids = nuids;
-  fakecred.ngids = ngids;
-
-  flags = 0;
-  if (!diskfs_access (np, S_IREAD, &fakecred))
-    flags |= O_READ;
-  if (!diskfs_access (np, S_IEXEC, &fakecred))
-    flags |= O_EXEC;
-  if (!diskfs_access (np, S_IWRITE, &fakecred)
-      && !S_ISDIR (np->dn_stat.st_mode)
-      && !diskfs_check_readonly ())
-    flags |= O_WRITE;
-  
-  err = diskfs_create_protid (diskfs_make_peropen (np, flags, MACH_PORT_NULL),
-			      uids, nuids, gids, ngids, &newpi);
-  
-  diskfs_nput (np);
-  ports_port_deref (pt);
-  
-  if (!err)
-    {
-      *file = ports_get_right (newpi);
-      *filetype = MACH_MSG_TYPE_MAKE_SEND;
-    }
   return err;
 }
-
-  
