@@ -1,6 +1,6 @@
 /* Check the existence of mach devices
 
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
 
    Written by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -20,7 +20,6 @@
 
 #include <stdio.h>
 #include <argp.h>
-#include <error.h>
 
 #include <hurd.h>
 #include <mach.h>
@@ -39,14 +38,13 @@ static const char *doc = "Test for the existance of mach device DEVNAME..."
 int
 main (int argc, char **argv)
 {
-  error_t err;
-  mach_port_t device_master;
   /* Print devices found?  (otherwise only exit status matters)  */
   int print = 1;
   /* If printing, print all devices on the command line that are found. 
      Otherwise, just the first one found is printed.  */
   int all = 1;
   int found_one = 0;
+  mach_port_t device_master = MACH_PORT_NULL;
 
   /* Parse our options...  */
   error_t parse_opt (int key, char *arg, struct argp_state *state)
@@ -66,7 +64,14 @@ main (int argc, char **argv)
 	  all = 0; break;
 
 	case ARGP_KEY_ARG:
-	  err = device_open (device_master, 0, arg, &device);
+	  if (device_master == MACH_PORT_NULL)
+	    {
+	      err = get_privileged_ports (0, &device_master);
+	      if (err)
+		argp_failure (state, 3, err, "Can't get device master port");
+	    }
+
+	  err = device_open (device_master, D_READ, arg, &device);
 	  if (err == 0)
 	    /* Got it.  */
 	    {
@@ -85,7 +90,7 @@ main (int argc, char **argv)
 	    }
 	  else if (err != ED_NO_SUCH_DEVICE)
 	    /* Complain about unexpected errors.  */
-	    error (0, err, "%s", arg);
+	    argp_failure (state, 0, err, "%s", arg);
 	  break;
 
 	default:
@@ -95,10 +100,6 @@ main (int argc, char **argv)
     }
   const struct argp argp = { options, parse_opt, args_doc, doc };
 
-  err = get_privileged_ports (0, &device_master);
-  if (err)
-    error (3, err, "Can't get device master port");
-
   /* Parse our arguments.  */
   argp_parse (&argp, argc, argv, 0, 0, 0);
 
