@@ -1,6 +1,6 @@
 /* Show where a file exists
 
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
 
    Written by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -32,16 +32,17 @@
 #include <hurd/fs.h>
 #include <hurd/store.h>
 
-char *argp_program_version = STANDARD_HURD_VERSION (storeinfo);
+const char *argp_program_version = STANDARD_HURD_VERSION (storeinfo);
 
 static struct argp_option options[] =
 {
-  {"kind",        'k', 0, 0, "Print the type of store behind FILE"},
+  {"type",        't', 0, 0, "Print the type of store behind FILE"},
+  {"flags",       'f', 0, 0, "Print the flags associated with FILE's store"},
   {"name",        'n', 0, 0, "Print the name of the store behind FILE"},
   {"blocks",      'b', 0, 0, "Print the number of blocks in FILE"},
   {"block-size",  'B', 0, 0, "Print the block size of FILE's store"},
   {"size",        's', 0, 0, "Print the size, in bytes, of FILE"},
-  {"runs",        'r', 0, 0, "Print the runs of blocks in FILE"},
+  {"block-list",  'l', 0, 0, "Print the blocks that are in FILE"},
   {"children",	  'c', 0, 0, "If the store has children, show them too"},
   {"dereference", 'L', 0, 0, "If FILE is a symbolic link, follow it"},
   {"prefix",      'p', 0, 0, "Always print `FILE: ' before info"},
@@ -50,26 +51,27 @@ static struct argp_option options[] =
 };
 static char *args_doc = "FILE...";
 static char *doc = "Show information about storage used by FILE..."
-"\vWith no FILE arguments, the file attached to standard \
-input is used.  The fields to be printed are separated by colons, in this \
-order: PREFIX: KIND: NAME: BLOCK-SIZE: BLOCKS: SIZE: RUNS.  If the store is a \
-composite one and --children is specified, children are printed on lines \
-following the main store, indented accordingly.  By default, all \
-fields, and children, are printed.";
+"\vWith no FILE arguments, the file attached to standard"
+" input is used.  The fields to be printed are separated by colons, in this"
+" order: PREFIX: TYPE (FLAGS): NAME: BLOCK-SIZE: BLOCKS: SIZE: BLOCK-LIST."
+"  If the store is a composite one and --children is specified, children"
+" are printed on lines following the main store, indented accordingly."
+"  By default, all fields, and children, are printed.";
 
 /* ---------------------------------------------------------------- */
 
 /* Things we can print about a file's storage.  */
 #define W_SOURCE	0x01
-#define W_KIND		0x02
+#define W_TYPE		0x02
 #define W_NAME		0x04
 #define W_BLOCKS	0x08
 #define W_BLOCK_SIZE	0x10
 #define W_SIZE		0x20
 #define W_RUNS		0x40
 #define W_CHILDREN	0x80
+#define W_FLAGS		0x100
 
-#define W_ALL		0xFF
+#define W_ALL		0x1FF
 
 /* Print a line of information (exactly what is determinted by WHAT)
    about store to stdout.  LEVEL is the desired indentation level.  */
@@ -113,9 +115,9 @@ print_store (struct store *store, int level, unsigned what)
       putchar (' ');
     }
 
-  pstr (store->class->name,W_KIND);
+  pstr (store->class->name,W_TYPE);
 
-  if (store->flags && (what & W_KIND))
+  if ((store->flags & ~STORE_INACTIVE) && (what & W_FLAGS))
     {
       int t = 0;		/* flags tested */
       int f = 1;
@@ -142,14 +144,14 @@ print_store (struct store *store, int level, unsigned what)
       pf (STORE_ENFORCED, "enf");
       pf (STORAGE_MUTATED, "mut");
 
-      if (store->flags & ~t)
+      if (store->flags & ~(t | STORE_INACTIVE))
 	/* Leftover flags. */
 	{
 	  if (! f)
 	    putchar (';');
-	  printf ("0x%x", store->flags);
+	  printf ("0x%x", store->flags & ~(t | STORE_INACTIVE));
 	}
-      putchar ('(');
+      putchar (')');
     }
 
   pstr (store->name,       W_NAME);
@@ -165,9 +167,10 @@ print_store (struct store *store, int level, unsigned what)
 	  if (i > 0)
 	    putchar (',');
 	  if (store->runs[i].start < 0)
-	    printf ("[%ld]", store->runs[i].length);
+	    /* A hole */
+	    printf ("@+%ld", store->runs[i].length);
 	  else
-	    printf ("%ld[%ld]", store->runs[i].start, store->runs[i].length);
+	    printf ("%ld+%ld", store->runs[i].start, store->runs[i].length);
 	}
     }
 
@@ -202,7 +205,7 @@ main(int argc, char *argv[])
 	  if (what == 0)
 	    what = W_ALL;
 
-	  err = store_create (file, 0, 0, &store);
+	  err = store_create (file, STORE_INACTIVE, 0, &store);
 	  if (err)
 	    error (4, err, source);
 
@@ -216,12 +219,13 @@ main(int argc, char *argv[])
 	case 'p': print_prefix = 1; break;
 	case 'P': print_prefix = 0; break;
 
-	case 'k': what |= W_KIND; break;
+	case 't': what |= W_TYPE; break;
+	case 'f': what |= W_FLAGS; break;
 	case 'n': what |= W_NAME; break;
 	case 'b': what |= W_BLOCKS; break;
 	case 'B': what |= W_BLOCK_SIZE; break;
 	case 's': what |= W_SIZE; break;
-	case 'r': what |= W_RUNS; break;
+	case 'l': what |= W_RUNS; break;
 	case 'c': what |= W_CHILDREN; break;
 
 	case ARGP_KEY_NO_ARGS:
