@@ -1,6 +1,6 @@
 /* Hurdish w
 
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
 
    Written by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -43,13 +43,13 @@
 
 #include "psout.h"
 
-#define DEFAULT_FMT_STRING "%^%user %tty %from %login %idle %what"
+#define DEFAULT_FMT_STRING "%^%user %tty %from %login %idle %pid %what"
 
 extern char *canon_host (char *host);
 extern char *shared_domain (char *host1, char *host2);
 extern char *localhost ();
 
-char *argp_program_version = STANDARD_HURD_VERSION (w);
+const char *argp_program_version = STANDARD_HURD_VERSION (w);
 
 #define OA OPTION_ARG_OPTIONAL
 
@@ -93,6 +93,14 @@ struct w_hook
 #define W_PSTAT_IDLE	(PSTAT_USER_BASE << 2)
 #define W_PSTAT_LOGIN	(PSTAT_USER_BASE << 3)
 
+static ps_flags_t
+w_deps (ps_flags_t flags)
+{
+  if (flags & W_PSTAT_IDLE)
+    flags |= PSTAT_TTY;
+  return flags;
+}
+
 static ps_flags_t
 w_fetch (struct proc_stat *ps, ps_flags_t need, ps_flags_t have)
 {
@@ -122,24 +130,33 @@ w_fetch (struct proc_stat *ps, ps_flags_t need, ps_flags_t have)
 	      if (sd)
 		*sd = '\0';
 	    }
-
-	  have |= W_PSTAT_HOST;
 	}
+      have |= W_PSTAT_HOST;
     }
 
   if (need & W_PSTAT_IDLE)
-    {
-      struct stat stat;
-      struct ps_tty *tty = ps->tty;
+    if (have & PSTAT_TTY)
+      {
+	struct stat stat;
+	struct ps_tty *tty = ps->tty;
 
-      hook->idle.tv_usec = 0;
-      if (io_stat (tty->port, &stat) == 0)
-	{
-	  hook->idle.tv_sec = now.tv_sec - stat.st_mtime;
-	  if (hook->idle.tv_sec > 0)
+	hook->idle.tv_usec = 0;
+	if (! tty)
+	  {
+	    hook->idle.tv_sec = 0;
 	    have |= W_PSTAT_IDLE;
-	}
-    }
+	  }
+	else
+	  {
+	    if (io_stat (tty->port, &stat) == 0)
+	      {
+		hook->idle.tv_sec = now.tv_sec - stat.st_atime;
+		have |= W_PSTAT_IDLE;
+	      }
+	  }
+      }
+    else if (ps->inapp & PSTAT_TTY)
+      ps->inapp |= W_PSTAT_IDLE;
 
   if (need & W_PSTAT_USER)
     if (ps_user_uname_create (hook->utmp.ut_name, &hook->user) == 0)
@@ -205,21 +222,22 @@ w_get_host (struct proc_stat *ps, char **host, unsigned *host_len)
 {
   struct w_hook *hook = ps->hook;
   *host = hook->host;
-  *host_len = strlen (*host);
+  *host_len = *host ? strlen (*host) : 0;
 }
 const struct ps_getter w_host_getter =
 {"host", W_PSTAT_HOST, w_get_host};
 
 extern error_t ps_emit_past_time (), ps_emit_string (), ps_emit_minutes ();
 extern error_t ps_emit_user_name ();
-extern int ps_cmp_times (), ps_cmp_strings ();
+extern int ps_cmp_times (), ps_cmp_strings (), ps_cmp_unames ();
+extern int ps_nominal_string ();
 const struct ps_fmt_spec _w_specs[] =
 {
   {"User", 0, 8,   -1,0, &w_uname_getter,ps_emit_string, ps_cmp_strings},
-  {"Name", 0, 20,  -1,0, &w_user_getter, ps_emit_user_name,ps_cmp_strings},
+  {"Name", 0, 16,  -1,0, &w_user_getter, ps_emit_user_name,ps_cmp_unames,ps_nominal_string},
   {"Login","Login@", -7,  -1,0,&w_login_getter,ps_emit_past_time,ps_cmp_times},
-  {"From", 0, 16,  -1,0, &w_host_getter, ps_emit_string, ps_cmp_strings},
-  {"Idle", 0, -5,  -1,0, &w_idle_getter, ps_emit_minutes,ps_cmp_times},
+  {"From", 0, 14,  -1,0, &w_host_getter, ps_emit_string, ps_cmp_strings, ps_nominal_string},
+  {"Idle", 0, -5,  -1,PS_FMT_FIELD_COLON_MOD, &w_idle_getter, ps_emit_minutes,ps_cmp_times},
   {"What=args"},
   {0}
 };
@@ -240,6 +258,9 @@ add_utmp_procs (struct proc_stat_list *procs, struct utmp *u)
   int pos;
   struct proc_stat *ps;
 
+  if (u->ut_type == DEAD_PROCESS || !*u->ut_name)
+    return;			/* Unused entry.  */
+
   strncpy (tty, u->ut_line, sizeof u->ut_line);
   tty[sizeof u->ut_line] = '\0'; /* Ensure it's '\0' terminated. */
 
@@ -302,33 +323,6 @@ add_utmp_procs (struct proc_stat_list *procs, struct utmp *u)
 	   tty, pid < 0 ? "pgrp" : "pid", pid < 0 ? -pid : pid);
 }
 
-/* Read into PROCS from the utmp file called NAME.  */
-static void
-read_utmp_procs (struct proc_stat_list *procs, char *name)
-{
-  int rd, fd;
-  struct utmp buf[64];
-
-  fd = open (name, O_RDONLY);
-  if (fd < 0)
-    error (9, errno, "%s", name);
-
-  while ((rd = read (fd, buf, sizeof (buf))) > 0)
-    {
-      struct utmp *u = buf;
-      while (rd >= sizeof (*u))
-	{
-	  if (*u->ut_name && *u->ut_line)
-	    /* An active entry.  */
-	    add_utmp_procs (procs, u);
-	  rd -= sizeof (*u);
-	  u++;
-	}
-    }
-
-  close (fd);
-}
-
 static void
 uptime (struct proc_stat_list *procs)
 {
@@ -370,21 +364,34 @@ void
 main(int argc, char *argv[])
 {
   error_t err;
+  struct utmp *ut;
   struct ps_context *context;
   int output_width = -1;
   char *fmt_string = DEFAULT_FMT_STRING, *sort_key_name = NULL;
   int sort_reverse = 0, print_heading = 1, show_entries = 1, show_uptime = 1;
   int squash_bogus_fields = 1, squash_nominal_fields = 1;
+  int default_output = 1;	/* should show standard `all users' list.  */
   struct proc_stat_list *procs;
 #if 0
   char *tty_names = 0;
   unsigned num_tty_names = 0;
-  struct idvec *only_uids = make_idvec (), *not_uids = make_idvec ();
 #endif
-  struct ps_user_hooks ps_hooks = { 0, w_fetch, w_cleanup };
+  uid_t *users = 0;
+  size_t num_users = 0;
+  struct ps_user_hooks ps_hooks = { w_deps, w_fetch, w_cleanup };
 
   int has_hook (struct proc_stat *ps) { return ps->hook != 0; }
 
+  int keep_users (struct proc_stat *ps)
+    {
+      int i;
+      struct w_hook *h = ps->hook;
+      for (i = 0; i < num_users; i++)
+	if (users[i] == h->user->uid)
+	  return 1;
+      return 0;
+    }
+
   /* Parse our options...  */
   error_t parse_opt (int key, char *arg, struct argp_state *state)
     {
@@ -401,6 +408,19 @@ main(int argc, char *argv[])
 	case 'w': output_width = arg ? atoi (arg) : 0; break;
 
 	case ARGP_KEY_ARG:
+	  num_users++;
+	  users = realloc (users, num_users * sizeof (*users));
+	  if (! users)
+	    argp_failure (state, 5, ENOMEM, "%s", arg);
+	  else if (isdigit (*arg))
+	    users[num_users - 1] = atoi (arg);
+	  else
+	    {
+	      struct passwd *pw = getpwnam (arg);
+	      if (! pw)
+		argp_failure (state, 6, 0, "%s: Unknown user", arg);
+	      users[num_users - 1] = pw->pw_uid;
+	    }
 	  break;
 
 	default:
@@ -425,18 +445,25 @@ main(int argc, char *argv[])
   /* Parse our options.  */
   argp_parse (&argp, argc, argv, 0, 0, 0);
 
-  read_utmp_procs (procs, _PATH_UTMP);
+  /* Read the utmp file.  */
+  setutent ();
+  while (ut = getutent ())
+    add_utmp_procs (procs, ut);
+  endutent ();
 
   /* Keep only processes that have our hooks attached.  */
   proc_stat_list_filter1 (procs, has_hook, 0, 0);
 
+  if (num_users > 0)
+    proc_stat_list_filter1 (procs, keep_users, W_PSTAT_USER, 0);
+
   if (show_uptime)
     uptime (procs);
 
   if (show_entries)
     psout (procs, fmt_string, 0, &w_specs, sort_key_name, sort_reverse,
 	   output_width, print_heading,
-	   squash_bogus_fields, squash_nominal_fields);
+	   squash_bogus_fields, squash_nominal_fields, 0);
 
   exit(0);
 }
