@@ -1,5 +1,5 @@
 /* Hurd version
-   Copyright (C) 1996 Free Software Foundation, Inc.
+   Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    Written by Thomas Bushnell, n/BSG.
 
    This file is part of the GNU Hurd.
@@ -21,10 +21,12 @@
 /* See sh-version.sed for duplicates of this information. */
 
 #ifndef HURD_VERSION
-#define HURD_VERSION "0.1"
+#define HURD_VERSION "0.2"
 #endif
 
-/* The standard way to print versions for --version */
-#define STANDARD_HURD_VERSION(s) #s " - GNU Hurd-" HURD_VERSION
+/* The standard way to print versions for --version.  */
+#define _SHV_SEP "; "
+#define STANDARD_HURD_VERSION(s, extra...) \
+  #s " (GNU Hurd" _SHV_SEP ##extra ") " HURD_VERSION
 
    
